#!/usr/bin/perl -I/usr/local/lib/bs/bin
#    Big Sister network monitor
#    Copyright (C) 1998  Thomas Aeby
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#=============================================================================
#
$main::Usage	  = "[-D level] [-r rule_file] [-t loop_time] [-s state_dir] [-l logfile] [-b basedir]";
#
#=============================================================================
@main::options = ( "r:s", "t:i", "s:s\@", "l:s@" );

use common;
proginit();

use strict;
use FileHandle;
use bb_evgen;

my $loop_delay = 15*60;
my $log_delay = 30;

my $dl = $main::dl;

&bb_evgen::init;

$ENV{PATH} = "$main::root/bin:".$ENV{PATH};

my @state_dirs = ("$main::root/www/logs");
my $rule_file = "$main::root/adm/bb_event_generator.cfg";
$bb_evgen::pager = "$main::root/bin/log_mail";
$bb_evgen::page_user = "alarm";
$main::on_unix = Platforms::isunix();

($rule_file=$main::opt_r) if( $main::opt_r );
(@state_dirs=@main::opt_s) if( @main::opt_s );
my @log_files = @main::opt_f;
unless( @log_files ) {
    if( -d "$main::root/www/logs/history" ) {
	@log_files = "$main::root/www/logs/history/display.history";
    }
}
($loop_delay=5*60) unless( @log_files );
($loop_delay=$main::opt_t) if( $main::opt_t );

$bb_evgen::rule_file = $rule_file;
$bb_evgen::group_file = "$main::root/www/grouping";
$bb_evgen::state_file = "$main::root/adm/bb_event_generator.state";
@bb_evgen::state_dirs = @state_dirs;
$bb_evgen::override_dir = "$main::root/www/logs/override";

&bb_evgen::load_rules || exit 1;
&bb_evgen::load_groups;
&bb_evgen::load_state;
&bb_evgen::read_state;
&bb_evgen::open_logs( @log_files );

&background unless( $dl );

my $next_poll = time+$loop_delay;
my $sleep;

while( 1 ) {
    &bb_evgen::check_rulefile;
    &bb_evgen::check_logs;
    &bb_evgen::check_alarms;
    if( time>=$next_poll ) {
	&bb_evgen::check_state;
	$next_poll += $loop_delay;
    }
    &bb_evgen::flush_state;
    $sleep = $next_poll-time;
    $sleep = $log_delay if( $sleep>$log_delay );
    sleep $sleep if( $sleep>0 );
    my $max=0;
    my $pid;
    if( $main::on_unix ) {
	while( ($pid=Platform::bswait_nohang() != -1) && ($max++<10) ) {
	    print "child $pid terminated\n" if( $dl>2 );
	}
    }
}
