package bb_evgen;

#=============================================================================
#
# Id	  : @(#) $Id: bb_evgen.pm,v 1.12 2000/01/02 15:27:47 aeby Exp $
# Package : bb
# File	  : bb_evgen
# Author  : Thomas Aeby
# Date	  : 22.10.97
#
#
# $Log: bb_evgen.pm,v $
# Revision 1.12  2000/01/02 15:27:47  aeby
# limit comment in SNMP traps to 150 chars
#
# Revision 1.11  1999/12/05 15:11:44  aeby
# added SNMP trap generation to bb_event_generator
#
# Revision 1.10  1999/10/24 12:38:36  aeby
# place alarms for checks listed in the ignore list in the 'new_alarms' list
# and keep them there until either the alarm condition or the ignore mode
# has gone. bswebalarm: show them as acknowledged
#
# Revision 1.9  1999/06/17 20:15:41  aeby
# on win32 systems use Platform::background_process for running the notifier
#
# Revision 1.8  1999/06/12 15:16:38  aeby
# tested on WinNT and fixed flaky implementations
#
# Revision 1.7  1999/06/11 17:58:34  aeby
# page(), raise(), ack(): include time in alarms sent
#
# Revision 1.6  1999/06/09 18:34:24  aeby
# do not use Socket module
#
# Revision 1.5  1999/06/07 18:47:03  aeby
# use log() to log event_generator events.
# assume checks are up when undefined (e.g. when used in "check=...")
#
# Revision 1.4  1999/06/07 18:14:19  aeby
# *** empty log message ***
#
# Revision 1.3  1999/05/09 17:14:43  aeby
# send "ack" alarm when receiving an acknowledgement
#
# Revision 1.2  1999/05/09 15:12:35  aeby
# completed support for alarm acknowledges
#
#
#=============================================================================

# check: condition met -> do not clear delayed alarm
# delay: delay in minutes
# repeat: warn every x minutes until alarm is cleared ("reminder")
# norepeat: do not raise another alarm for the same host/check within x minutes
# keep: keep alarm for at least x minutes even if it is cleared
# prio: alarm priority
# repeatprio: reminder priority
# mail: comma separated list of mail recipients

use strict;
use FileHandle;
use snmp;

sub init {
    @bb_evgen::state_dirs = ();
    @bb_evgen::logs = ();
    $bb_evgen::pager = "";
    $bb_evgen::pager_user = "";
    $bb_evgen::rule_file = "";
    $bb_evgen::rule_file_stat = "";
    $bb_evgen::group_file = "";
    $bb_evgen::group_file_stat = "";
    $bb_evgen::state_file = "";

    %bb_evgen::numstates = ( "red", 0, "yellow", 1, "purple", 2, "green", 3, "never", -1 );
    %bb_evgen::textstates = ( 0, "red", 1, "yellow", 2, "purple", 3, "green", -1, "never" );

    %bb_evgen::previous_state = ();
    @bb_evgen::new_alarms = ();
    @bb_evgen::old_alarms = ();
    @bb_evgen::alarms = ();
    %bb_evgen::ign_acks = ();
    %bb_evgen::up = ();

    $bb_evgen::minute = 60;
    $bb_evgen::dirty = 1;
}



sub check_rulefile {
    print "check_rulefile()\n" if( $main::dl );
    &bb_evgen::load_rules if( (stat($bb_evgen::rule_file))[9] != $bb_evgen::rule_file_stat );
    &bb_evgen::load_groups if( (stat($bb_evgen::group_file))[9] != $bb_evgen::group_file_stat );
}


sub check_state {
    print "check_state()\n" if( $main::dl );
    &bb_evgen::read_state;
}


sub common_check {
    my( $type, $i, $queue, $alarm ) = @_;

    if( (defined $alarm->{"ack_until"}) && ($alarm->{"ack_until"} < time) && ($alarm->{"ack_until"} > 0) ) {
	delete $alarm->{"ack_until"};
	delete $alarm->{"ack_user"};
	delete $alarm->{"ack_mode"};
	delete $alarm->{"ack_text"};
	$bb_evgen::dirty = 1;
    }
    if( (defined $alarm->{"ack_mode"}) && ($alarm->{"ack_mode"} eq "del") ) {
	splice( @$queue, $i, 1 );
	$bb_evgen::dirty = 1;
    }
}



sub check_alarms {
    my( $i, $var, $now );
    print "check_alarms()\n" if( $main::dl );
    $now = time;
    for( $i=$#bb_evgen::new_alarms; $i>=0; $i-- ) {
	$var = $bb_evgen::new_alarms[$i];
	&common_check( "new", $i, \@bb_evgen::new_alarms, $var );
	if( ($$var{"delay"}<$now) && ($$var{"postpone"}<$now) &&
	    (! defined $var->{"ack_mode"}) &&
	    (! defined $bb_evgen::ign_acks{($var->{"host"}).".".($var->{"item"})}) ) {
	    splice( @bb_evgen::new_alarms, $i, 1 );
	    $var->{"_ignored"} = 0;
	    &bb_evgen::alarm( $var );
	}
    }
    for( $i=$#bb_evgen::alarms; $i>=0; $i-- ) {
	$var = $bb_evgen::alarms[$i];
	&common_check( "active", $i, \@bb_evgen::alarms, $var );
	if( $$var{"_next_repetition"} && ($$var{"_next_repetition"} < $now) ) {
	    &bb_evgen::alarm( $var, 1 );
	}
	if( $$var{"_dequeue"} && ($$var{"_dequeue"}<$now) 
	    && ($bb_evgen::up{$$var{"host"}.".".$$var{"item"}}) ) {
	    &bb_evgen::clear_alarm( $i );
	}
    }
    for( $i=$#bb_evgen::old_alarms; $i>=0; $i-- ) {
	$var = $bb_evgen::old_alarms[$i];
	&common_check( "old", $i, \@bb_evgen::old_alarms, $var );
	if( $$var{"norepeat"} <= $now ) {
	    &main::log( "notice", "forgetting old alarm: ".$$var{"host"}.".".$$var{"item"}." ".$$var{"text"} );
	    print "\x07forgetting old alarm: ".$$var{"host"}.".".$$var{"item"}." ".$$var{"text"}."\n" if( $main::dl );
	    splice( @bb_evgen::old_alarms, $i, 1 );
	    $bb_evgen::dirty = 1;
	}
    }
    while( ($i,$var) = each %bb_evgen::ign_acks ) {
	if( ($var->{"eol"}>0) && ($var->{"eol"}<time) ) {
	    delete $bb_evgen::ign_acks{$i};
	    $bb_evgen::dirty = 1;
	}
    }
}



sub check_logs {
    my( $log );
    my( $log_file, $fh, $pos, $open, $ino, $buff ); 
    my( $newbuff, $line, $prev, $newnum );

    print "check_logs()\n" if( $main::dl );
    foreach $log (@bb_evgen::logs) {
	( $log_file, $fh, $pos, $open, $ino, $buff ) = @$log; 
	print "checking $log_file\n" if( $main::dl>3 );
	if( $open ) {
	    seek( $fh, $pos, 0 );
	    read( $fh, $newbuff, 200000 );
	    print "read at pos $pos: $newbuff\n" if( $main::dl>7 );
	    $buff .= $newbuff;
	    $pos = tell($fh);
	    if( $ino != ((stat($log_file))[1]) ) {
		print "log file $log_file was replaced\n" if( $main::dl>4 );
		&main::log( "info", "log file changed" );
		close( $fh );
		$open = 0;
	    }
	}
	unless( $open ) {
	    print "(re-)opening log file $log_file\n" if( $main::dl>4 );
	    if( open( $fh, "<$log_file" ) ) {
		$open = 1;
		$ino = ((stat($log_file))[1]);
		read( $fh, $newbuff, 200000 );
		$buff .= $newbuff;
		$pos = tell($fh);
	    }
	}
	while( $buff =~ /^([^\n]*)\n/ ) {
	    $buff = $';
	    $line = $1;
	    print "line: $line\n" if( $main::dl>5 );
	    if( $line =~ /^([^\.]+)\.([^:]+): ([a-z]+)->([a-z]+)(.*)$/ ) {
		my( $host, $item, $old, $new, $text ) = ($1,$2,$3,$4,$5);
		$newnum = $bb_evgen::numstates{$new};
		$prev = $bb_evgen::previous_state{$host.".".$item};
		print "$host.$item $prev->$newnum($new)\n" if( $main::dl>5 );
		if( $newnum != $prev ) {
		    ($prev = $newnum) unless( defined $prev );
		    &bb_evgen::raise( $host, $item, $bb_evgen::numstates{$old}, 
			$newnum, $new." ".$text );
		    $bb_evgen::dirty = 1;
		}
		$bb_evgen::previous_state{$host.".".$item} = $newnum;
	    }
	    elsif( $line =~ /^([^\.]+)\.([^:]+): ACK ([^\s]+) ([^\s]+) ([^\s]+)(.*)$/ ) {
		my( $host, $item, $duration, $user, $what, $text ) = ($1,$2,$3,$4,$5,$6);
		my( $i, $ntime );

		print "got ack for $host.$item from $user for $duration: $what: $text\n" if( $main::dl>2 );
		&main::log( "notice", "got ack for $host.$item from $user for $duration: $what: $text" );
		my $time;
		if( $duration eq "forever" ) {
		    $time = -1;
		    $ntime = time+365*24*3600;
		}
		elsif( $duration =~ /^(\d+):(\d+)$/ ) {
		    $ntime = $time = time+(($1*60)+$2)*60;
		}
		else {
		    $ntime = $time = time+3600;
		    $duration = "unspecified";
		}
		foreach $i (@bb_evgen::alarms, @bb_evgen::new_alarms, @bb_evgen::old_alarms) {
		    next unless( ($i->{"host"} eq $host) && ($i->{"item"} eq $item) );
		    $i->{"ack_user"} = $user;
		    $i->{"ack_until"} = $time;
		    $i->{"ack_mode"} = $what;
		    $i->{"ack_text"} = $text;
		    ack( $i );
		}
		if( $what eq "ign" ) {
		    $bb_evgen::ign_acks{$host.".".$item} = {
			"eol" => $time,
			"host" => $host,
			"item" => $item,
			"user" => $user,
			"text" => $text,
			"until" => $time,
			"ack_user" => $user,
			"ack_mode" => $what,
			"ack_text" => $text,
			"ack_until" => $time
			};
		    ack( $bb_evgen::ign_acks{$host.".".$item} );
		}
		$bb_evgen::dirty = 1;
	    }
	}
	@$log = ( $log_file, $fh, $pos, $open, $ino, $buff ); 
    }
}



sub load_state {
    my( $alrmvar, $txt );
    %bb_evgen::previous_state = ();
    %bb_evgen::up = ();
    open( F, "<".$bb_evgen::state_file ) || return;
    while( <F> ) {
	chomp;
	if( /^status ([^\s]+) (.*)$/ ) {
	    $bb_evgen::previous_state{$1} = $2;
	}
	elsif( /^up ([^\s]+) (.*)$/ ) {
	    $bb_evgen::up{$1} = $2;
	}
	elsif( /^(.*alarms|.*acks) (.*)/ ) {
	    $alrmvar = $1;
	    $txt = $2;
	    my %alarm = ();
	    foreach (split("\x00",$txt)) {
		if( /^([^=]+)=(.*)$/ ) {
		    $alarm{$1} = $2;
		}
	    }
	    unless( defined $alarm{"id"} ) {
		$alarm{"id"} = $alarm{"host"}.".".$alarm{"item"};
	    }
	    $alarm{"queue"} = $alrmvar;
	    if( $alrmvar eq "alarms" ) {
		push( @bb_evgen::alarms, \%alarm );
	    }
	    elsif( $alrmvar eq "new_alarms" ) {
		push( @bb_evgen::new_alarms, \%alarm );
	    }
	    elsif( $alrmvar eq "old_alarms" ) {
		push( @bb_evgen::old_alarms, \%alarm );
	    }
	    elsif( $alrmvar eq "ign_acks" ) {
		$bb_evgen::ign_acks{($alarm{"host"}).".".($alarm{"item"})} = \%alarm;
	    }
	}
    }
    close F;
}



sub flush_state {
    my( $what, $status, $fh );

    return unless( $bb_evgen::dirty );
    $fh = FileHandle->new();
    open( $fh, ">".$bb_evgen::state_file.".tmp" );
    while( ($what,$status) = each %bb_evgen::previous_state ) {
	print $fh "status $what $status\n";
    }
    while( ($what,$status) = each %bb_evgen::up ) {
	print $fh "up $what ".($status?1:0)."\n";
    }
    &print_alarms( $fh, "alarms", @bb_evgen::alarms );
    &print_alarms( $fh, "new_alarms", @bb_evgen::new_alarms );
    &print_alarms( $fh, "old_alarms", @bb_evgen::old_alarms );
    &print_alarms( $fh, "ign_acks", values %bb_evgen::ign_acks );
    close $fh;
    rename( $bb_evgen::state_file.".tmp", $bb_evgen::state_file ) && ( $bb_evgen::dirty = 0 );
}


sub print_alarms {
    my( $fh, $what, @alarms ) = @_;
    my( $alarm );
    my( $var, $val );

    foreach $alarm (@alarms) {
	print $fh $what." ";
	while( ($var, $val) = each %$alarm ) {
	    print $fh "$var=$val\x00";
	}
	print $fh "\n";
    }
}



sub read_state {
    my( $dir, $file, $state, $prev, $color, $fulltext );

    print "read_state()\n" if( $main::dl>1 );
    foreach $dir( @bb_evgen::state_dirs ) {
	print "reading state dir $dir\n" if( $main::dl>2 );
	opendir( DIR, $dir );
	foreach $file (readdir( DIR )) {
	    next unless( (-f $dir."/".$file) && ($file =~ /\./) );
	    print "reading state file $file\n" if( $main::dl>8 );
	    open( F, "<$dir/$file" );
	    $fulltext = "";
	    while( <F> ) {
		$fulltext .= $_;
	    }
	    close F;
	    chomp $fulltext;
	    $fulltext =~ s/\n/\|\>/g;
	    $state = $fulltext;
	    $state =~ /^([a-z]*)/;
	    $state = $bb_evgen::numstates{$1};
	    print "state: $state\n" if( $main::dl>15 );
	    $prev = $bb_evgen::previous_state{$file};
	    next unless( defined $state );
	    if( $prev != $state ) {
		my( $host, $item ) = split( /\./, $file );
		($prev = $state) unless( defined $prev );
		&bb_evgen::raise( $host, $item, $prev, $state, $fulltext );
	    }
	    $bb_evgen::previous_state{$file} = $state;
	}
	close DIR;
    }
}
    

sub page {
    my( $pager, $host, $code, $text, $severity, $state, $fulltext, $msgmax, $mailhost, @users ) = @_;
    my( $short, $time );

    print "page( $pager, $host, $code, $text, $severity )\n" if( $main::dl>3 );
    if( $fulltext =~ /^.{0,6}\(([0-9]+)\)/ ) {
	$time = $1;
	$fulltext = $';
    }
    else {
	$time = time;
    }
    $fulltext =~ s/^.*?:\d\d [0-9]{4}\s//;
    if( $time ) {
	my( $sec, $min, $hour ) = localtime($time);
	$time = sprintf( " %02d.%02d", $hour, $min );
    }
    $short = $fulltext = $text." ".$state.$time.":".$fulltext;
    $short = substr( $short, 0, $msgmax ) if( $msgmax );
    ($severity = 50) unless( $severity ); 
    (@users = ($bb_evgen::page_user)) unless( @users );
    unless( $pager ) { $pager = $bb_evgen::pager; };
    &main::log( "notice", "sending page for $host with severity $severity by $pager to ".join(",",@users).": $short" );
    unless( $pager =~ /\s/ ) {
	my @cmd = ( $pager, "-S", $severity, "-s", $short, 
		"-h", $host, "-m", $fulltext,
		@users );
	if( $mailhost ) {
	    splice( @cmd, 1, 0, "-M", $mailhost );
	}
	if( Platforms::iswin32() ) {
	    unless( $cmd[0] =~ /\// ) {
		$cmd[0] = $main::root."/bin/".$cmd[0];
	    }
	    my $i;
	    for( $i=1; $i<=$#cmd; $i++ ) {
		$cmd[$i] =~ s/\"//g;
		$cmd[$i] = '"'.$cmd[$i].'"';
	    }
	    unshift( @cmd, "perl" );
	    Platform::background_process( "perl", join( " ", @cmd ) );
	}
	else {
	    print "executing ".join(" ",@cmd)."\n" if( $main::dl>1 );
	    system( @cmd );
	}
    }
}



sub raise {
    my( $host, $item, $old_state, $new_state, $text ) = @_;
    my( $addr, @matches, %vars, $match, $ref, $var, $cont, $up );

    print "raise( $host, $item, $old_state, $new_state, $text )\n" if( $main::dl>2 );
    &main::log( "info", "status for $host.$item changed from $old_state to $new_state" );
    if( $old_state<$new_state ) {
	my( $i );
	my $now = time;

	for( $i=$#bb_evgen::new_alarms; $i>=0; $i-- ) {
	    $var = $bb_evgen::new_alarms[$i];
	    if( (($$var{"host"} eq $host) && ($$var{"item"} eq $item)
		  && ($$var{"up"}<=$new_state))
		|| (! eval( $$var{"check"} ) || ($$var{"delay"}<$now)) ) {
	        splice( @bb_evgen::new_alarms, $i, 1 );
		$bb_evgen::dirty = 1;
	    }
	}
	for( $i=$#bb_evgen::alarms; $i>=0; $i-- ) {
	    $var = $bb_evgen::alarms[$i];
	    if( (($$var{"host"} eq $host) && ($$var{"item"} eq $item)
		  && ($$var{"up"}<=$new_state)) ) {
		$var->{"uptext"} = $text;
		$var->{"upstatus"} = $new_state;
	        &bb_evgen::clear_alarm( $i );
	    }
	}
    }
    my ($name,$aliases,$addrtype,$length,@addrs) = gethostbyname( $host );
    foreach $addr (@addrs) {
	$addr = join( ".", unpack( "C*", $addr ) );
    }
    %vars = ( "id", $host.time.$item, "host", $host, "item", $item, "text", $text, "time", time, "status", $new_state );
    foreach $match (@bb_evgen::rules) {
	 next unless( match_rule( $match, $host, $item, \@addrs ) );
	 next unless( eval($$match[3]) );
	 $ref = $$match[2];
	 while( ($var,$cont) = each %$ref ) {
	     $vars{$var} = $cont;
	 }
    }
    if( $main::dl>3 ) {
	print "matched rule: ";
	while( ($var,$cont) = each %vars ) {
	    print "$var=$cont, ";
	}
	print "\n";
    }
    unless( defined $vars{"down"} ) {
	    $vars{"down"} = 1;
    }
    unless( defined $vars{"up"} ) {
	    $vars{"up"} = $vars{"down"}+1;
    }
    if( $new_state>$old_state ) {
	$up = $new_state>=$vars{"up"};
    }
    else {
	$up = $new_state>$vars{"down"};
    }
    $bb_evgen::up{$host.".".$item} = $up;
    return if( ($old_state <= $new_state) || $up );
    if( $vars{"check"} ) {
	$vars{"check"} = &bb_evgen::compile( $vars{"check"}, \%vars );
    }
    else {
	$vars{"check"} = 1;
    }
    return unless( (eval $vars{"check"}) );
    if( grep( ($host eq $$_{"host"}) && ($item eq $$_{"item"}), 
	@bb_evgen::new_alarms, @bb_evgen::alarms, @bb_evgen::old_alarms ) ) {
	print "no alarm since alarm pending/norepeated\n" if( $main::dl>2 );
	return;
    }
    if( $vars{"keep"} ) {
	$vars{"keep"} = time + $bb_evgen::minute*$vars{"keep"};
    }
    unless( $vars{"repeatprio"} ) {
	$vars{"repeatprio"} = $vars{"prio"};
    }
    if( $vars{"postpone_to"} ) {
	if( $vars{"postpone_to"} =~ /^(\d+):(\d+)$/ ) {
	    my( $hour, $min ) = ($1,$2);
	    my $sec = ($hour*60+$min)*60;
	    my $now = time;
	    my $tod = $now%(3600*24);
	    my $timezone = (12-((localtime(12*3600))[2]))*3600;
	    $tod -= $timezone;
	    if( $sec>=$tod ) {
		$sec = $sec-$tod;
	    }
	    else {
		$sec = 24*3600+$sec-$tod;
	    }
	    $vars{"postpone"} = $sec/60;
	    print "time update: $sec\n";
	}
    }
    if( defined $bb_evgen::ign_acks{$host.".".$item} ) {
	$vars{"_ignored"} = 1;
    }
    if( $vars{"_ignored"} || $vars{"delay"} || $vars{"postpone"} ) {
	$vars{"delay"} = time+$bb_evgen::minute*$vars{"delay"} if( $vars{"delay"} );
	$vars{"postpone"} = time+$bb_evgen::minute*$vars{"postpone"} if( $vars{"postpone"} );
	$vars{"delay"} = 0 unless( $vars{"delay"} );
	$vars{"postpone"} = 0 unless( $vars{"postpone"} );
	if( $vars{"norepeat"} ) {
	    $vars{"norepeat"} = time + $bb_evgen::minute*$vars{"norepeat"};
	    $bb_evgen::dirty = 1;
	}
	$vars{"queue"} = "new_alarms";
	push( @bb_evgen::new_alarms, \%vars );
	$bb_evgen::dirty = 1;
	print "goes to delayed alarm queue\n" if( $main::dl>2 );
    }
    else {
	&bb_evgen::alarm( \%vars );
    }
}



sub alarm {
    my( $vars, $repeat ) = @_;

    print "\x07raising alarm: ".$$vars{"host"}.".".$$vars{"item"}." ".$$vars{"text"}."\n" if( $main::dl );
    &main::log( "notice", "raising alarm: ".$$vars{"host"}.".".$$vars{"item"}." ".$$vars{"text"} );
    unless( grep( $_ eq $vars, @bb_evgen::alarms ) ) {
	$vars->{"queue"} = "alarms";
	push( @bb_evgen::alarms, $vars );
	$bb_evgen::dirty = 1;
    }
    if( $$vars{"repeat"} ) {
	($$vars{"_next_repetition"} = time) unless( $$vars{"_next_repetition"} );
	$$vars{"_next_repetition"} += $$vars{"repeat"}*$bb_evgen::minute;
	$bb_evgen::dirty = 1;
    }
    return if( $vars->{"ack_mode"} && $repeat );
    &bb_evgen::page( $$vars{"pager"}, $$vars{"host"}, $$vars{"item"}, $$vars{"host"}.":".$$vars{"item"}, $repeat?$$vars{"repeatprio"}:$$vars{"prio"}, $repeat?"reminder":"down", $$vars{"text"}, $$vars{"msgmax"}, $$vars{"mailhost"}, split(",",$$vars{"mail"}) );
    if( $vars->{"trap"} ) {
	snmp::trap( 
	    $vars->{"trap"},
	    undef,
	    $vars->{"host"},
	    $repeat?"bsremind":"bsservicedown",
	    {
		"bshost" => [ "string", $vars->{"host"}],
		"bsservice" => [ "string", $vars->{"item"}],
		"bsstatus" => [ "string", $bb_evgen::textstates{$vars->{"status"}}],
		"bscomment" => [ "string", substr($vars->{"text"},0,150)],
		"bsseverity" => [ "int", $repeat?$$vars{"repeatprio"}:$$vars{"prio"} ]
	    } );
    }
}



sub ack {
    my( $vars ) = @_;

    my $text = $vars->{"ack_text"};
    my $ack = ($vars->{"ack_mode"})." by ".($vars->{"ack_user"}).": ";
    $text =~ s/^(.*?[0-9]:[0-9]{2} [0-9]{4}\s)/$1 $ack/; 
    &bb_evgen::page( $$vars{"pager"}, $$vars{"host"}, $$vars{"item"}, $$vars{"host"}.":".$$vars{"item"}, $$vars{"prio"}, "ack", $text, $$vars{"msgmax"}, $$vars{"mailhost"}, split(",",$$vars{"mail"}) );
    if( $vars->{"trap"} ) {
	snmp::trap( 
	    $vars->{"trap"},
	    undef,
	    undef,
	    "bsalarmack",
	    {
		"bshost" => [ "string", $vars->{"host"}],
		"bsservice" => [ "string", $vars->{"item"}],
		"bscomment" => [ "string", $vars->{"ack_text"}],
		"bsackmode" => [ "string", $vars->{"ack_mode"}],
		"bsackuser" => [ "string", $vars->{"ack_user"}]
	    } );
    }
}



sub clear_alarm {
    my( $i ) = @_;
    my( $vars );

    $vars = $bb_evgen::alarms[$i];
    print "\x07clearing alarm: ".$$vars{"host"}.".".$$vars{"item"}." ".$$vars{"text"}."\n" if( $main::dl );
    &main::log( "info", "clearing alarm: ".$$vars{"host"}.".".$$vars{"item"}." ".$$vars{"text"} );
    if( $$vars{"keep"} && ($$vars{"keep"}>time) ) {
	$$vars{"_dequeue"} = $$vars{"keep"};
	print "keep alarm since 'keep' was defined\n" if( $main::dl>1 );
	return;
    }
    splice( @bb_evgen::alarms, $i, 1 );
    $bb_evgen::dirty = 1;
    if( $$vars{"norepeat"} && ($$vars{"norepeat"}>time) ) {
	$vars->{"queue"} = "old_alarms";
	push( @bb_evgen::old_alarms, $vars );
    }
    &bb_evgen::page( $$vars{"pager"}, $$vars{"host"}, $$vars{"item"}, $$vars{"host"}.":".$$vars{"item"}, $$vars{"prio"}, "up", "", $$vars{"msgmax"}, $$vars{"mailhost"}, split(",",$$vars{"mail"}) );
    if( $vars->{"trap"} ) {
	snmp::trap( 
	    $vars->{"trap"},
	    undef,
	    $vars->{"host"},
	    "bsserviceup",
	    {
		"bshost" => [ "string", $vars->{"host"}],
		"bsservice" => [ "string", $vars->{"item"}],
		"bsstatus" => [ "string", $bb_evgen::textstates{$vars->{"upstatus"}}],
		"bscomment" => [ "string", substr($vars->{"uptext"},0,100)],
		"bsseverity" => [ "int", $$vars{"prio"} ]
	    } );
    }
}


sub compile {
    my( $check, $vars ) = @_;
    my( $s );
    my %days = ( "Sun", 0, "Mon", 1,
		 "Tue", 2, "Wed", 3,
		 "Thu", 4, "Fri", 5,
		 "Sat", 6 );

    $s = "";
    print "compile( $check )\n" if( $main::dl>3 );
    while( $check =~ /\$([a-z]+)/ ) {
	if( $vars ) {
	    $s .= $`.$$vars{$1};
	}
	else {
	    $s .= $`."\$vars{'$1'}";
	}
	$check = $';
    }
    $check = $s.$check;
    $s = "";
    while( $check =~ /(daytime|weekday)[\s\t]+([0-9:a-z\-,]+)/i ) {
	$s .= $`;
	$check = $';
	my ($cmd,$args) = ($1,$2);
	if( $cmd eq "daytime" ) {
	    unless( $args =~ /^(\d+):(\d+)-(\d+):(\d+)$/ ) {
		print STDERR "$main::prog: wrong time format (should be hh:mm-hh:mm) in daytime in line $.\n";
		$args = 0;
	    }
	    else {
		my $start = ($1*60+$2)*60;
		my $end = ($3*60+$4)*60;
		my $timezone = (12-((localtime(12*3600))[2]))*3600;
		if( $end < $start ) {
		    $args = "((((time-($timezone))%86400)>=$start) || (((time-($timezone))%86400)<$end))";
		}
		else {
		    $args = "((((time-($timezone))%86400)>=$start) && (((time-($timezone))%86400)<$end))";
		}
  	    }
	}
	elsif( $cmd eq "weekday" ) {
	    my @days = (0,0,0,0,0,0,0);
	    my $expr;
	    foreach $expr (split(",",$args)) {
		if( defined $days{$expr} ) {
		    $days[$days{$expr}] = 1;
		}
		elsif( ($expr =~ /(.*)-(.*)/) && defined $days{$1} && defined $days{$2} ) {
		    my ($start,$end) = ($days{$1},$days{$2});
		    $days[$start] = 1;
		    while( $start != $end ) {
			$start++;
			$start=0 if( $start>6 );
			$days[$start] = 1;
		    }
		}
		else {
		    print STDERR "$main::prog: wrong day expression in weektime in line $.\n";
		}
	    }
	    $args = "((".join(",",@days).")[((localtime)[6])])";
	}
	$s .= $args;
    }
    $check = $s.$check;
    $s = "";
    while( $check =~ /([a-z:_0-9\-]*)\.([a-z:_0-9\-]+)/ ) {
	$s .= $`."((! defined \$bb_evgen::up{\"$1.$2\"}) || \$bb_evgen::up{\"$1.$2\"})";
	$check = $';
    }
    $check = $s.$check;
    print "compiled to $check\n" if( $main::dl>4 );
    return $check;
}



sub match_rule {
    my( $rule, $host, $item, $addrs ) = @_;
    my( $rhost, $ritem, $vars ) = @$rule;
    my( $ref );

    return(0) if( ($ritem ne "*") && ($ritem ne $item) );
    if( substr( $rhost, 0, 1 ) eq "[" ) {
	$rhost = substr( $rhost, 1, length($rhost)-2 );
	return(0) unless( grep( substr( $_.".", 0, length($rhost)+1 ) eq $rhost.".", @$addrs ) );
    }
    elsif( substr( $rhost, 0, 1 ) eq "@" ) {
	$rhost = substr( $rhost, 1 );
	$ref = $bb_evgen::groups{$host};
	return(0) unless( $ref && grep( $_ eq $rhost, @$ref ) );
	print "$host is in group $rhost\n" if( $main::dl>6 );
    }
    else {
	return(0) if( ($rhost ne "*") && ($rhost ne $host) );
    }
    return 1;
}




sub load_groups {
    my( $host, @groups, $ref );
    my( $fh );

    $bb_evgen::group_file_stat = (stat($bb_evgen::group_file))[9];
    unless( -f "$bb_evgen::group_file" ) {
	print STDERR "cannot load group file $bb_evgen::group_file\n";
	return 0;
    }
    print "loading group file $bb_evgen::group_file\n" if( $main::dl>1 );
    %bb_evgen::groups = ();
    $fh = FileHandle->new();
    open( $fh, "<$bb_evgen::group_file" );
    while( <$fh> ) {
	chomp;
	next unless( /^join (.*)$/ );
	($host,@groups) = split( " ", $1 );
	unless( $ref = $bb_evgen::groups{$host} ) {
	    $ref = $bb_evgen::groups{$host} = [];
	}
	push( @$ref, @groups );
    }
    close( $fh );
}




sub load_rules {
    my( $var, $match, $fh, $chk, $pre_condition );
    my( $host, $item );

    $bb_evgen::rule_file_stat = (stat($bb_evgen::rule_file))[9];
    unless( -f "$bb_evgen::rule_file" ) {
	print STDERR "cannot load rule file $bb_evgen::rule_file\n";
	return 0;
    }
    print "loading rule file $bb_evgen::rule_file\n" if( $main::dl>1 );
    @bb_evgen::rules = ();
    $fh = FileHandle->new();
    open( $fh, "<$bb_evgen::rule_file" );
    while( <$fh> ) {
	chomp;
	next if( /^#/ || /^$/ );
	while( /\\$/ ) {
	    $chk = $_;
	    $_ = <$fh>;
	    last unless( $_ );
	    chomp;
	    $_ = substr($chk,0,length($chk)-1).$_;
	}
	if( /^([^\s\t]+)/ ) {
	    $match = $1;
	    $var = $';
	    if( ($match =~ /\{/) && ! ($match =~ /\}/) && ($var =~ /^(.*?\})/) ) {
		$match .= $1;
		$var = $';
	    }
	    if( $match =~ /\{(.*)\}/ ) {
		$match = $`.$';
		$pre_condition = bb_evgen::compile( $1 );
	    }
	    else {
		$pre_condition = 1;
	    }
	    my @tok = split( /\./, $match );
	    my $check = pop @tok;
	    $match = join( "_", @tok ).".".$check;
	}
	else {
	    next;
	}
	print "adding rule match: $match, precondition: $pre_condition, vars: $var\n" if( $main::dl>5 );
	my %vars = ();
	while( $var ) {
	    $var =~ s/^[\s\t]+//;
	    if( ($var =~ /^([a-z0-9_]+)[\s\t]*=[\s\t]*\"([^\"]*)\"/)
	    	|| ($var =~ /^([a-z0-9_]+)[\s\t]*=[\s\t]*([^\s\t]+)/) ) {
		$vars{$1} = $2;
		$var = $';
	    }
	    else {
		last;
	    }
	}
	if( $vars{"down"} ) {
	    $vars{"down"} = $bb_evgen::numstates{$vars{"down"}};
	}
	if( $vars{"up"} ) {
	    $vars{"up"} = $bb_evgen::numstates{$vars{"up"}};
	}
	foreach $chk (split( ";", $match )) {
	    $chk =~ /^(.*)\.([^\.]*)$/;
	    ($host, $item) = ($1, $2);
	    push( @bb_evgen::rules, [ $host, $item, \%vars, $pre_condition ] );
	}
    }
    close $fh;
    return 1;
}


sub open_logs {
    my @log_files = @_;
    my $log_file;
    my( $open, $ino );

    @bb_evgen::logs = ();
    foreach $log_file (@log_files) {
	 my( $fh, $pos ) = (FileHandle->new(), 0);
	 my $open = 0;
	 if( open( $fh, "<$log_file" ) ) {
	     $open = 1;
	     seek( $fh, 0, 2 );
	     $pos = tell($fh);
	     $ino = ((stat($log_file))[1]);
	}
	push( @bb_evgen::logs, [ $log_file, $fh, $pos, $open, $ino, "" ] );
    }
}

1;
