package bbdisp;


use Socket;
use strict;
use FileHandle;

my @buttons = ( "red.gif", "yellow.gif", "purple.gif", "green.gif" );
my @bground = ( "bkg-red.gif", "bkg-yellow.gif", "bkg-purple.gif", "bkg-green.gif" );
my @state_text = ( "red", "yellow", "purple", "green" );

sub new
{
    my ($this, $cfg) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);
    $self->init;
    $self->{"cfg_file"} = $cfg;
    $self->{"dir"} = "$main::root/www";
    $self->{"groupfile"} = $self->{"dir"}."/grouping";
    $self->{"gifs"} = "gifs";
    $self->{"mtime"} = -1;
    $self->{"bb_modload"} = 0;
    $self->read_file( $self->{"cfg_file"} );
    return $self;
}


sub init {
    my( $self ) = @_;
    my( @new );
 
    $self->{"groups"} = {};
    $self->{"dirty"} = {};
    $self->{"groups_by_sub"} = {};
    $self->{"subs_by_group"} = {};
    $self->{"group_titles"} = {};
    $self->{"states"} = {} unless( $self->{"states"} );
    $self->{"states_text"} = {} unless( $self->{"states_text"} );
    $self->{"pages"} = {};
    $self->{"frames"} = {};
    $self->{"ordered_pages"} = [];
    $self->{"isdirty"} = 1;
    $self->{"rsync"} = [];
    $self->{"synctime"} = 0;
    $self->{"groupdirty"} = 1;
    $self->{"grouptime"} = time+15*60;
    $self->{"host_there"} = {};
    $self->{"group_status"} = {};
    $self->{"images"} = [];
    $self->{"autoconn"} = [];
    $self->{"options"} = {
	"NewLog" => 1,
	"BBLog" => 1
	};
    $self->{"defaultskin"} = $self->{"logskin"} = $self->{"skin"} = $self->read_skin( "default" );
    if( $self->{"autojoin_allhosts"} || $self->{"autojoin_all"} ) {
	my( $host, $states );
	$states = $self->{"states"};
	foreach $host (keys %$states) {
	    $self->join_group( $host, $self->{"autojoin_allhosts"} ) if( $self->{"autojoin_allhosts"} );
	    $self->join_group( $host, $self->{"autojoin_all"} ) if( $self->{"autojoin_all"} );
	}
    }
}


sub set_config {
    my( $self, $config ) = @_;

    $self->{"config"} = $config;
}


sub group_status {
    my( $self, $group ) = @_;

    return( $state_text[$self->{"group_status"}->{$group}] );
}


sub set {
    my( $self, $group, $item, $state, $text ) = @_;

    if( $self->{"autojoin_new"} && ! defined $self->{"groups_by_sub"}->{$group} ) {
	$self->join_group( $group, $self->{"autojoin_new"} );
    }
    if( $self->{"autojoin_allhosts"} && ! $self->{"host_there"}->{$group} ) {
	$self->join_group( $group, $self->{"autojoin_allhosts"} );
	$self->{"host_there"}->{$group} = 1;
    }
    $self->_set( "states", $group, $item, $state, $text );
}



sub _set {
    my( $self, $base, $group, $item, $state, $text ) = @_;
    my( $grps, $nstat, $update, $textupdate );

    print STDERR "bbdisp::_set( $base, $group, $item, $state )\n" if( $main'dl>2 );
    $text =~ s/^[^\s]+\s// if( $text );
    if( $base eq "states" ) {
	if(defined $self->{"states_text"}->{$group}->{$item}) {
	    my $prevtext = $self->{"states_text"}->{$group}->{$item};
	    my $newtext = $text;
	    $newtext =~ s/^.*?:\d\d \d{2,4}//;
	    $prevtext =~ s/^.*?:\d\d \d{2,4}//;
	    $textupdate = $prevtext ne $newtext;
	}
	$self->{"states_text"}->{$group}->{$item} = $text;
    }
    $update = (defined $self->{$base}->{$group}->{$item}) && ($self->{$base}->{$group}->{$item} != $state);
    if( ($base eq "states") && ($self->{"options"}->{"ImmediateHTML"} || $update || $textupdate || ! -f $self->{"dir"}."/html/".$group.".$item.html" )) {
    	$self->write_html_status( $group, $item, $state, $text );
    }
    if( $update && ($base eq "states")  ) {
	$self->log( "$group.$item: ".$state_text[$self->{$base}->{$group}->{$item}]."->".$state_text[$state]." $text" );
    }
    return if( (defined $self->{$base}->{$group}->{$item}) && ($self->{$base}->{$group}->{$item} == $state) );
    $self->{$base}->{$group}->{$item} = $state;
    $self->{"isdirty"} = 1;
    $grps = $self->{"groups_by_sub"}->{$group};
    foreach $group (@$grps) {
	$nstat = (defined $self->{"groups"}->{$group}->{$item})?($self->{"groups"}->{$group}->{$item}):100;
	next if( $state == $nstat );
	if( $state < $nstat ) {
	    $self->_set( "groups", $group, $item, $state );
	    if( $self->{"group_status"}->{$group} > $state ) {
		$self->{"group_status"}->{$group} = $state;
	    }
	}
	else {
	    $self->{"dirty"}->{$group} = 1;
	}
    }
}



sub log {
    my( $self, $txt ) = @_;

    open( LOG, ">>".$self->{"dir"}."/logs/history/display.history" );
    print LOG $txt."\n";
    close LOG;
}



sub write_html_status {
    my( $self, $group, $item, $state, $text, $date ) = @_;
    my( %vars, $skin, $html );

    $skin = $self->{"logskin"};
    $vars{"HOST"} = $group;
    $vars{"ITEM"} = $item;
    if( $text =~ /^(.*?:\d\d \d{2,4})/ ) {
	$vars{"DATE"} = $1;
	$text = $';
    }
    $text =~ s/\|\>/\n/g;
    $html = "";
    if ($text =~ /^(.*?)\&html/s) {
	# Client has told us it understands html
	($text,$html) = ($1,$');
	$html =~ s/\&html//g;
    }
    $text =~ s/\</\&lt;/g;
    $text =~ s/\>/\&gt;/g;
    $text =~ s/^[\s\n]+//;
    $text =~ s/\n/\<BR\>/g;
    $text =~ s/\@/\@\@/g;
    $text =~ s/\&(green|yellow|red|purple)/\@\(lamp.sub:$1\)\@/g;
    $text = $text.$html;
    $skin->{"_BASE"} = "..";
    $vars{"STATUS"} = $state_text[$state];
    $vars{"TEXT"} = $text;
    $vars{"TIME"} = localtime( time );
    $vars{"BGROUND"} = $self->get_skin( $skin, $bground[$state] );
    $vars{"TEXT"} = $self->replace_vars( $skin, \%vars, $skin->{"loghtml_text.proto"} );
    $self->replace_vars_file( $skin, \%vars, $skin->{"loghtml.proto"}, $self->{"dir"}."/html/".$group.".$item.html" );
    delete $skin->{"_BASE"};
}



sub recompute {
    my( $self ) = @_;
    my( @dirty, $pass, $dirty, $group, $sub, $key, $state, $val, $ref, $minstat );

    $dirty = $self->{"dirty"};
    $pass = 0;
    while( ($pass++<20) && (@dirty = keys %$dirty) ) {
	while( @dirty ) {
	    my %cont;
	    $group = shift @dirty;
	    delete $$dirty{$group};
	    next if( defined $self->{"states"}->{$group} );
	    $self->{"groups"}->{$group} = {};
	    $ref = $self->{"subs_by_group"}->{$group};
	    foreach $sub (@$ref) {
		if( defined $self->{"states"}->{$sub} ) {
		    $val = $self->{"states"}->{$sub};
		} else {
		    $val = $self->{"groups"}->{$sub};
		}
		while( ($key, $state) = each %$val ) {
		    if( (! defined $cont{$key}) || ($state < $cont{$key}) ) {
			$cont{$key} = $state;
		    }
		}
	    }
	    $minstat = 3;
	    while( ($key, $state) = each %cont ) {
		($minstat = $state) if( $state < $minstat );
		$self->_set( "groups", $group, $key, $state );
	    }
	    $self->{"group_status"}->{$group} = $minstat;
	}
    }
}



sub read_file {
    my( $self, $file ) = @_;
    my( $cmd, $args, @args, $title, $group, $ref, $page, %args );
    my( $table, $pageattr );

    print STDERR "bbdisp::read_file( $file )\n" if( $main'dl>1 );
    open( IN, "<$file" );
    $self->init;
    $_ = <IN>;
    chomp;
outer: while( defined $_ ) {
	if( /^#/ || /^$/ ) {
	    ; # do nowt
	}
	elsif( /^%(Groups|Logskin|Page|skin|title|refto|table|ref|itemref|Rsync|Pager|Autojoin|image|Autoconn|Frameset|include|Option)(|\s+.*)$/ ) {
	    ($cmd, $args) = ($1,$2);
	    $args =~ s/^\s+//;
	    @args = split( /\s+/, $args );
	    print "executing $cmd( ".join(",",@args)." )\n" if( $main::dl>5 );
	    if( $cmd eq "Groups" ) {
		while( <IN> ) {
		    next if( /^#/ || /^$/ );
		    next outer if( /^%/ );
		    $title = "";
		    if( /\((.*)\)/ ) {
			$title = $1;
			$_ = $`.$';
		    }
		    my @args = split( /\s+/ );
		    $args = shift @args;
		    $title = $args unless( $title );
		    $args =~ s/\./_/g;
		    $self->displayname( $args, $title );
#		    $self->{"group_titles"}->{$args} = $title;
#		    $self->{"groups"}->{$args} = {};
#		    $self->{"dirty"}->{$args} = 1;
#		    $self->{"groups_by_sub"}->{$args} = \@args;
		    foreach $group (@args) {
			$self->join_group( $args, $group );
#			unless( defined $self->{"subs_by_group"}->{$group} ) {
#			    $self->{"subs_by_group"}->{$group} = $ref = [];
#			}
#			else {
#			    $ref = $self->{"subs_by_group"}->{$group};
#			}
#			push( @$ref, $args );
		    }
		}
	    }
	    elsif( $cmd eq "Autoconn" ) {
		my( $ref, $host, $addr );

		$ref = $self->{"autoconn"};
		foreach $host (@args) {
		    my ($name,$aliases,$addrtype,$length,@addrs) = gethostbyname($host);
		    foreach $addr (@addrs) {
			push( @$ref, [ $addr, $host ] );
		    }
		}
	    }
	    elsif( $cmd eq "Autojoin" ) {
		my( $what, $group ) = @args;

		if( $what eq "new" ) {
		    $self->{"autojoin_new"} = $group;
		} elsif( $what eq "all" ) {
		    $self->{"autojoin_all"} = $group;
		} elsif( $what eq "all_hosts" ) {
		    $self->{"autojoin_allhosts"} = $group;
		}
	    }
	    elsif( $cmd eq "Rsync" ) {
		my( $host, $prefix, @groups ) = @args;
		my( $rsync, $port );

		$port = 1984;
		if( $host =~ /:/ ) {
		     $port = $host;
		     $host =~ s/:.*//;
		     $port =~ s/.*://;
		     unless( $port =~ /^\d+$/ ) { $port = getservbyname( $port, "tcp" ); }
		}
		$rsync = $self->{"rsync"};
		unless( inet_aton($host) ) {
		    print STDERR "$main'prog: WARNING: host $host appearing in line $. of config is not a valid host name\n"; 
		}
		push( @$rsync, {
		    "host" => $host,
		    "groups" => \@groups,
		    "port" => $port,
		    "prefix" => ($prefix eq "none")?"":$prefix
		    } );
	    }
	    elsif( $cmd eq "Frameset" ) {
		my( $name, $top, $title ) = @args;
		unless( $top =~ /\.htm/ ) { $top .= ".html"; }
		my $ref = [ @{$self->{"ordered_pages"}} ];
		$self->{"frames"}->{$name} = {
		    "name" => $name,
		    "top" => $top,
		    "title" => $title,
		    "pages" => $ref
		    };
	    }
	    elsif( $cmd eq "Option" ) {
		my $option;
		foreach $option (@args) {
		    my $val = 1;
		    if( $option =~ /^(\-|\+)(.*)$/ ) {
			$option = $2;
			$val = ($1 eq "+");
		    }
		    $self->{"options"}->{$option} = $val;
		}
	    }
	    elsif( $cmd eq "Page" ) {
		my( $name, $title, $template ) = @args;
		my( $file, $ref );

		unless( $name =~ /\.htm/ ) { $name .= ".html"; }
		%args = ( "title", "auto" );
		$page = [];
		$title =~ s/_/ /g;
		$self->{"pages"}->{$name} = $page;
		$ref = $self->{"ordered_pages"};
		push( @$ref, $name );
		$file = $self->{"dir"};
		$file .="/" if( $file );
		$file .= $name;
		push( @$page, $pageattr = {
		    "type" => "page",
		    "title" => $title,
		    "template" => $template,
		    "file" => $file,
		    "name" => $name,
		    "skin" => $self->{"skin"},
		    "color" => 3
		    } );
	    }
	    elsif( $cmd eq "ref" ) {
		unless( $page ) { 
		    print STDERR "$main'prog: %$cmd statement not allowed without former %Page statement in line $.\n";
		    next;
		}
		push( @$page, {
		    "type" => "ref",
		    "name" => $args[0]
		    } );
	    }
	    elsif( $cmd eq "refto" ) {
		if( ($args[0] eq "none") || (! $args[0]) ) {
		    delete $args{"refto"};
		}
		elsif( $args[0] eq "clear" ) {
		    delete $args{"row-refto"};
		}
		elsif( $args[1] ) {
		    $args{"row-refto"} = {} unless( defined $args{"row-refto"} );
		    $ref = $args{"row-refto"};
		    $$ref{$args[0]} = $args[1];
		}
		elsif( $args[0] eq "self" ) {
		    $args{"refto"} = "#";
		}
		else {
		    $args{"refto"} = $args[0];
		    unless( $args{"refto"} =~ /\.htm/ ) { $args{"refto"} .= ".html"; };
		    $args{"refto"} .= "#";
		}
	    }
	    elsif( ($cmd eq "skin") || ($cmd eq "Logskin") ) {
		my $skin;
		my $read;

		foreach $read ("default", @args) {
		    $skin = $self->read_skin( $read, $skin );
		}
		$self->{"\L$cmd"} = $skin;
	    }
	    elsif( $cmd eq "title" ) {
		if( ($args[0] eq "none") || (! $args[0]) ) {
		    delete $args{"title"};
		}
		else {
		    $args{"title"} = $args[0];
		}
	    }
	    elsif( $cmd eq "itemref" ) {
		if( ($args[0] eq "none") || (! $args[0]) ) {
		    delete $args{"itemref"};
		}
		else {
		    $args{"itemref"} = $args[0];
		}
	    }
	    elsif( $cmd eq "include" ) {
		my $inc;

		foreach $inc( @args ) {
		    unless( substr( $inc, 0, 1 ) eq "/" ) {
			$inc = "$main::root/".$inc;
		    }
		    unless( -f $inc ) {
			print STDERR "$main'prog: WARNING: file $inc not found (line $.)"; 
		    }
		    push( @$page, {
			"type" => "include",
			"file" => $inc
			} );
		}
	    }
	    elsif( $cmd eq "table" ) {
		foreach $table (@args) {
		    my( %tab, %clone );
		    $tab{"title"} = $args{"title"} if( $args{"title"} );
		    if( $table =~ /^(\++)(.*)$/ ) {
			$table = $2;
			$tab{"depth"} = length( $1 );
		    }
		    push( @$page, {
			"type" => "ref",
			"name" => $table
			} );
		    $tab{"group"} = $table;
		    $tab{"refto"} = $args{"refto"} if( $args{"refto"} );
		    $tab{"itemref"} = $args{"itemref"} if( $args{"itemref"} );
		    $tab{"type"} = "table";
		    if( $ref = $args{"row-refto"} ) {
			%clone = %$ref;
			$tab{"row-refto"} = \%clone;
		    }
		    push( @$page, \%tab );
		}
	    }
	    elsif( $cmd eq "image" ) {
		my( $file, $img ) = @args;
		my( %image, $images, %clone );
		
		if( -f $file ) {;
		} elsif( -f "$main::root/$file" ) {
		    $file = "$main::root/$file";
		} elsif( -f "$main::root/adm/$file" ) {
		    $file = "$main::root/adm/$file";
		}
		else {
		    print STDERR "$main'prog: no file called $file at line $.\n";
		    next;
		}
		require "display_map.pm" unless( defined &display_map::new );
		$image{"type"} = "image";
		$image{"image"} = display_map->new( $self, $file );
		$image{"imgfile"} = $img;
		$image{"refto"} = $args{"refto"} if( $args{"refto"} );
		if( $ref = $args{"row-refto"} ) {
		    %clone = %$ref;
		    $image{"row-refto"} = \%clone;
		}
		push( @$page, \%image );
		$images = $self->{"images"};
		push( @$images, $image{"image"} );
	    }
	    elsif( $cmd eq "Pager" ) {
		$self->{"pager"} = $args[0];
	    }
	    else {
		print STDERR "$main'prog: unrecognized statement $cmd at line $.\n";
	    }
	}
	else {
	    print STDERR "$main'prog: unrecognized statement at line $.\n";
	}
	$_ = <IN>;
	chomp if( $_ );
    }
    close IN;
    $self->read_groups() if( $self->{"options"}->{"KeepGroups"} );
}



sub read_groups {
    my( $self ) = @_;
    my $file = $main::root."/www/grouping";
    open( IN, "<$file" ) || return;
    while( <IN> ) {
	chomp;
	if( /^displayname ([^\s]+)\s(.*)$/ ) {
	    $self->displayname( $1, $2 );
	}
	elsif( /^join ([^\s]+) (.*)$/ ) {
	    my( $sub, $groups ) = ($1,$2),
	    my $group;

	    foreach $group (split( /\s+/, $groups )) {
		$self->join_group( $sub, $group );
	    }
	}
    }
    close IN;
}



sub displayname {
    my( $self, $group, $title ) = @_;

    return if( $self->{"group_titles"}->{$group} eq $title );
    $self->{"group_titles"}->{$group} = $title;
    $self->{"dirty"}->{$group} = 1;
    $self->{"isdirty"} = 1;
    $self->{"groupdirty"} = 1;
}


sub join_group {
    my( $self, $sub, $group ) = @_;
    my( $subs );

    unless( defined $self->{"subs_by_group"}->{$group} ) {
	#new group
	$self->{"groups"}->{$group} = {};
	$self->{"subs_by_group"}->{$group} = [];
    }
    $subs = $self->{"subs_by_group"}->{$group};
    return if( grep( $_ eq $sub, @$subs ) );
    push( @$subs, $sub );
    unless( defined $self->{"groups_by_sub"}->{$sub} ) {
	$self->{"groups_by_sub"}->{$sub} = [];
    }
    $subs = $self->{"groups_by_sub"}->{$sub};
    push( @$subs, $group );
    $self->{"dirty"}->{$group} = 1;
    $self->{"isdirty"} = 1;
    my $i;
    foreach $i ($sub, $group) {
	unless( $self->{"group_titles"}->{$i} ) {
	    my $title = $i;
	    $title =~ s/_/\./g;
	    $self->{"group_titles"}->{$i} = $title;
	}
    }
    $self->{"groupdirty"} = 1;
    if( $self->{"autojoin_all"} && ($group ne $self->{"autojoin_all"}) ) {
	$self->join_group( $sub, $self->{"autojoin_all"} );
    }
}



sub leave_group {
    my( $self, $sub, $group ) = @_;
    my( $subs );

    return unless( (defined $self->{"groups"}->{$group}) && (defined $self->{"groups_by_sub"}->{$sub}) );
    $subs = $self->{"subs_by_group"}->{$group};
    @$subs = grep( $_ ne $sub, @$subs );
    $subs = $self->{"groups_by_sub"}->{$sub};
    @$subs = grep( $_ ne $group, @$subs );
    unless( @$subs ) {
	if( defined $self->{"states"}->{$sub} ) {
	    delete $self->{"states"}->{$sub};
	}
	else {
	    delete $self->{"groups"}->{$sub};
	}
    }
    $self->{"dirty"}->{$group} = 1;
    $self->{"isdirty"} = 1;
    $self->{"groupdirty"} = 1;
}




sub compute {
    my( $self ) = @_;
    my( $mtime );

    print STDERR "bbdisp::compute\n" if( $main'dl>1 );
    $self->recompute;
}


sub check_config {
    my( $self ) = @_;
    my( $mtime );

    $mtime = 0;
    $mtime = (stat($self->{"cfg_file"}))[9];
    return unless($mtime);
    if( $mtime != $self->{"mtime"} ) {
	$self->read_file( $self->{"cfg_file"} );
	$self->{"mtime"} = $mtime;
	$self->recompute;
    }
}


sub need_update {
    my( $self ) = @_;

    return( 1 );
}


sub update {
    my( $self ) = @_;
    my( $mtime, $page, $cont, $page_ref );

    print STDERR "bbdisp::update\n" if( $main'dl>1 );
    $self->compute if( $self->{"isdirty"} );
    $page_ref = $self->{"pages"};
    if( time>$self->{"synctime"} ) {
	print STDERR "syncing since time since last sync is ".(time-$self->{"synctime"})."\n" if( $main::dl>6 );
	$self->rsync;
    }
    if( time>$self->{"grouptime"} && $self->{"groupdirty"} ) {
	$self->{"grouptime"} = time+5*60;
	$self->group_write;
	$self->{"groupdirty"} = 0;
    }
    return unless( $self->{"isdirty"} );
    while( ($page,$cont) = each %$page_ref ) {
	$self->create_page( $page, $cont );
    }
    while( ($page,$cont) = each %{$self->{"frames"}} ) {
	$self->create_frameset( $cont->{"name"}, $cont->{"top"},
	    $cont->{"title"}, $cont->{"pages"} );
    }
    $self->{"isdirty"} = 0;
}



sub rsync {
    my( $self ) = @_;
    my( $rsync, $upd, $bbd, $base );

    print STDERR "bbdisp::rsync\n" if( $main::dl>1 );
    $rsync = $self->{"rsync"};
    return unless( @$rsync );
    unless( $self->{"bb_modload"} ) {
	$self->{"bb_modload"} = 1;
	push( @INC, $main::root."uxmon" );
	require Monitor::bb unless( defined &Monitor::bb::new );
    }
    foreach $upd (@$rsync) {
	my( %states, %comments, %joins, $text );
	print STDERR "bbdisp::rsync: updateing ".$upd->{"host"}."\n" if( $main::dl>2 );
	$bbd = Monitor::bb->new( $upd->{"host"}, 10, $upd->{"port"} );
	my( @groups, $group, $arg_groups, $syncgroups, @syncgroups, $syncgroup );
	$arg_groups = $upd->{"groups"};
	foreach $group (@$arg_groups) {
	    my( $groups ) = $self->{"subs_by_group"}->{$group};
	    push( @groups, @$groups );
	}
	foreach $group (@groups) {
	    if( defined $self->{"states"}->{$group} ) { $base = "states"; }
	      else { $base = "groups"; }
	    my( $item, $state, $items );
	    $items = $self->{$base}->{$group};
	    while( ($item,$state) = each %$items ) {
		$states{$upd->{"prefix"}.$group.".".$item} = $state;
		if( $base eq "states" ) {
		    $text = $self->{"states_text"}->{$group}->{$item};
		    if( $text =~ /^\((\d+)\)[^:]+:\d\d:\d\d\s+\d{4,4}(.*)$/ ) {
			$text = "(proxy delay: ".(time-$1)."s)".$2;
		    }
		    $comments{$upd->{"prefix"}.$group.".".$item} = $text;
		}
	    }
	}
	my( $i );
	for( $i=0; $i<=$#groups; $i++ ) {
	    $group = $groups[$i];
	    print STDERR "bbdisp::rsync: sending grouping/display info for $group\n" if( $main::dl>5 );
	    $syncgroups = $self->{"groups_by_sub"}->{$group};
	    my @syncgroups = ();
	    foreach $syncgroup (@$syncgroups) {
		push( @syncgroups, $upd->{"prefix"}.$syncgroup );
		push( @groups, $syncgroup ) unless( grep( $_ eq $syncgroup, @groups ) );
	    }
	    $joins{"+".$upd->{"prefix"}.$group} = \@syncgroups;
	    $joins{"D".$upd->{"prefix"}.$group} = [ $self->{"group_titles"}->{$group} ];
	}
	$bbd->report( \%states, \%comments, \%joins );
	undef $bbd;
    }
}
		
	

sub group_write {
    my( $self ) = @_;;
    my( $hash, $group, $name, $ref );
    my( $value );

    print STDERR "bbdisp::group_write()\n" if( $main::dl>2 );
    open( OUT, ">".$self->{"groupfile"} );
    $ref = $self->{"config"};
    while( ($name,$value) = each %$ref ) {
	print OUT "set $name=$value\n";
    }
    $hash = $self->{"group_titles"};
    while( ($group,$name) = each %$hash ) {
        print OUT "displayname $group $name\n";
    }
    $hash = $self->{"groups_by_sub"};
    while( ($group,$ref) = each %$hash ) {
	print OUT "join $group ".join(" ",@$ref)."\n" if( @$ref );
    }
    close OUT;
}




sub create_frameset {
    my( $self, $name, $top, $title, $pages ) = @_;
    my( $skin, %vars, $cont, $i, $ref, $proto, $pa );

    print "create_frameset()\n" if( $main::dl > 1 );
    $skin = $self->{"skin"};
    return unless( defined $skin->{"frameset.proto"} );
    $vars{"FRAMESET"} = $name;
    $title =~ s/_/ /g;
    $vars{"TITLE"} = $title;
    $vars{"TOP"} = $top;

    $cont = "";
    $ref = $self->{"pages"};
    foreach $i (@$pages) {
	$pa = $ref->{$i};
	$pa = $$pa[0];
	$proto = $skin->{"frameset_contents.proto"};
	$title = $pa->{"title"};
	$title =~ s/_/ /g;
	$proto =~ s/\@TITLE\@/$title/g;
	$proto =~ s/\@PAGE\@/$pa->{"name"}/g;
	$proto =~ s/\@COLOR\@/$state_text[$pa->{"color"}]/g;
	$cont .= $proto;
    }

    $vars{"CONTENTS"} = $cont;
    $vars{"EXPIRES"} = Platform::strftime("%a, %d %h %Y %T GMT", time + 300);
    $self->replace_vars_file( $skin, \%vars, $skin->{"frameset.proto"}, $self->{"dir"}."/".$name.".html" );

    $i = 1;
    if( defined $skin->{"frame0.proto"} ) { $i=0; }
    while( defined $skin->{"frame$i.proto"} ) {
	$self->replace_vars_file( $skin, \%vars, $skin->{"frame$i.proto"}, $self->{"dir"}."/".$name."$i.html" );
	$i++;
    }
}




sub create_page {
    my( $self, $name, $elements ) = @_;
    my( $file, $template, $pageattr, %vars, $element, $type, $skin, $i );

    print STDERR "bbdisp::create_page( $name )\n" if( $main'dl > 1 );
    $pageattr = $$elements[0];
    $template = $$pageattr{"template"};
    $skin = $$pageattr{"skin"};
    if( $template && ! -f $template ) {
	$template = $main::root."/www/".$template;
    }
    if( -r $template ) {
	open( _F, "<$template" );
	$template = "";
	while( <_F> ) {
	    $template .= $_;
	}
	close( _F );
    }
    else {
	$template = $skin->{"template.proto"};
    }
    unless( $template ) {
	print STDERR "$main'prog: cannot open template file\n";
	return;
    }
    $file = $pageattr->{"file"};
    $vars{"TITLE"} = $$pageattr{"title"};
    $vars{"TEXT"} = "";
    $vars{"TIME"} = localtime;
    $vars{"EXPIRES"} = Platform::strftime("%a, %d %h %Y %T GMT", time + 300);
    foreach $element (@$elements) {
	$type = $$element{"type"};
	if( $type eq "ref" ) {
	    $i = $skin->{"label.proto"};
	    $i =~ s/\@NAME\@/$element->{"name"}/g;
	    $vars{"TEXT"} .= $i;
	}
	elsif( $type eq "table" ) {
	    $vars{"TEXT"} .= $self->create_table( $skin, $element, \%vars );
	}
	elsif( $type eq "image" ) {
	    $vars{"TEXT"} .= $self->create_image( $skin, $element, \%vars );
	}
	elsif( $type eq "include" ) {
	    open( _IN, "<".$$element{"file"} );
	    while( <_IN> ) {
		$vars{"TEXT"} .= $_;
	    }
	    close( _IN );
	}
    }
    $vars{"BGROUND"} = $skin->{$bground[$vars{"MIN"}]};
    $elements->[0]->{"color"} = $vars{"MIN"};
    $self->replace_vars_file( $skin, \%vars, $template, $file );
}



sub replace_vars_file {
    my( $self, $skin, $vars, $template, $file ) = @_;
    my( $tmp );

    print STDERR "replace_vars_file -> $file\n" if( $main::dl>3 );
    $tmp = $file.".$$";
    unless( open( OUT, ">$tmp" ) ) {
	print STDERR "$main'prog: cannot create state file $file\n";
	return;
    }
    my $txt = $self->replace_vars( $skin, $vars, $template );
    $txt =~ s/\@\@/\@/g;
    print OUT $txt;
    close OUT;
    unless( rename( $tmp, $file ) ) {
	if( Platforms::iswin32() ) {
	    unlink( $file );
	    rename( $tmp, $file );
	}
    }
    # just in case ...
    unlink( $tmp );
}



sub replace_vars {
    my( $self, $skin, $vars, $template ) = @_;

    print STDERR "replace_vars()\n" if( $main::dl>4 );
    my @repl = &_replace_vars( $self, $skin, $vars, $template, 0 );
    return ($repl[0].$repl[1]);
}


sub _replace_vars {
    my( $self, $skin, $vars, $template, $depth, $localvars ) = @_;
    my( $replaced );

    print STDERR "_replace_vars()\n" if( $main::dl>6 );
    $replaced = "";
    while( $template =~ /^(.*?)@/s ) {
	my $what = "\@";
	my $rest = $';
	my $pre = $1;
	my $next = substr( $rest, 0, 1 );
	if( substr( $pre, length($pre)-1 ) eq ")" ) {
	    $pre = substr( $1, 0, length($pre)-1 );
	    $what = ")\@";
	}
	elsif( ($next eq "(") || ($next eq "\@") ) {
	    $what .= $next;
	    $rest = substr( $rest, 1 );
	}
	$replaced .= $pre;
	$template = $rest;
	if( $what eq "\@\@" ) {
	    # @@ means '@'
	    $replaced .= "\@\@";
	    print STDERR "  found \@\n" if( $main::dl>7 );
	}
	elsif( $what eq "\@" ) {
	    # @variable@ -> replace variable by value
	    if( $template =~ /^([a-z0-9_\-\.]+)\@/i ) {
		$template = $';
		print STDERR "  found var $1\n" if( $main::dl>7 );
		$replaced .= $self->_one_var( $skin, $vars, $1, $depth, $localvars );
	    }
	    else {
		$replaced .= "\@";
		print STDERR "  found quote \@\n" if( $main::dl>7 );
	    }
	}
	elsif( $what eq ")\@" ) {
	    if( $depth ) {
		print STDERR "  found end_var )\@\n" if( $main::dl>7 );
		return( $replaced, $template );
	    }
	    print STDERR "  found quoted )\@\n" if( $main::dl>7 );
	    $replaced .= $what;
	}
	else {
	    my( $var );
	    print STDERR "  found nested \@(\n" if( $main::dl>7 );
	    ( $var, $template ) = $self->_replace_vars( $skin, $vars, $template, $depth+1, $localvars );
	    print STDERR "  var is $var\n" if( $main::dl>7 );
	    $replaced .= $self->_one_var( $skin, $vars, $var, $depth, $localvars );
	}
    }
    return( $replaced, $template );
}



sub _one_var {
    my( $self, $skin, $vars, $var, $depth, $localvars ) = @_;
    my( @args, $i );

    print STDERR "    _one_var( $var )\n" if( $main::dl>7 );
    @args = split( ":", $var );
    $var = shift @args;
    if( defined $localvars->{$var} ) {
	$var = $localvars->{$var};
    }
    elsif( defined $vars->{$var} ) {
	$var = $vars->{$var};
    }
    else {
	$var = $self->get_skin( $skin, $var );
    }
    $localvars = {};
    for( $i=0; $i<=$#args; $i++ ) {
	$localvars->{"ARG$i"} = $args[$i];
    }
    my( $a, $b ) = $self->_replace_vars( $skin, $vars, $var, $depth, $localvars );
    return ($a.$b);
}



sub create_image {
    my( $self, $skin, $image, $vars ) = @_;
    my( $text, $ref, $elem, $file, $rowrefto, $href );

    $image->{"image"}->create();
    if( defined $image->{"row-refto"} ) {
	$rowrefto = $image->{"row-refto"};
    }
    $file = $image->{"image"}->file;
    $file =~ s/^www\///;
    $text = $skin->{"image_map.proto"};
    $text =~ s/\@SOURCE\@/$file/g;
    $text =~ s/\@MAP\@/map/g;
    $text .= "\n<MAP name=\"map\">\n";
    $ref = $image->{"image"}->map;
    foreach $elem (@$ref) {
	$text .= "<AREA shape=\"rect\" coords=\"$$elem[0],$$elem[1],";
	$text .= ($$elem[0]+$$elem[2]).",".($$elem[1]+$$elem[3]);
	$text .= "\" alt=\"$$elem[4]\"";
	$href = "";
	$href = $image->{"refto"}.$$elem[4] if( $image->{"refto"} );
	$href = $rowrefto->{$$elem[4]} if( $rowrefto && $rowrefto->{$$elem[4]} );
	$text .= " href=\"$href\" " if( $href );
	$text .= ">\n";
    }
    $text .= "</MAP>\n";
}



sub create_table {
    my( $self, $skin, $table, $vars ) = @_;
    my( $text, $i, $ref, %items, $group, @groups, @items, @line, $item, $t, $proto );
    my( $refto, $rowrefto, $min, $itemref, $button, $title, $depth, @recurse, %lines, $host);

    $min = $#bground;
    $min = $$vars{"MIN"} if( defined $$vars{"MIN"} );
    print STDERR "bbdisp::create_table\n" if( $main'dl > 1 );
    $text = $skin->{"table.proto"};
    $title = $$table{"title"};
    $title = $self->{"group_titles"}->{$$table{"group"}} if( $title eq "auto" );
    $text =~ s/\@TITLE\@/$title/g;
    if( defined $$table{"refto"} ) {
	$refto = $$table{"refto"};
    }
    if( defined $$table{"row-refto"} ) {
	$rowrefto = $$table{"row-refto"};
    }
    if( defined $$table{"itemref"} ) {
	$itemref = $$table{"itemref"};
    }
    $ref = $self->{"subs_by_group"}->{$$table{"group"}};
    print "table $$table{group}\n" if( $main::dl>4 );
    return unless( $ref );
    @recurse = @$ref;
    $depth = $$table{"depth"};
    while( $depth ) {
	@groups = ();
	$depth--;
	foreach $group (@recurse) {
	    if( defined $self->{"subs_by_group"}->{$group} ) {
		$ref = $self->{"subs_by_group"}->{$group};
		push( @groups, @$ref );
	    }
	    else {
		push( @groups, $group );
	    }
	}
	@recurse = @groups;
    }
	    
    %items = ();
    @groups = ();
    foreach $i (@recurse) {
	if( defined $self->{"states"}->{$i} ) {
	    $group = $self->{"states"}->{$i};
	} else {
	    $group = $self->{"groups"}->{$i};
	}
	push( @groups, $i, $group );
	foreach $item (keys %$group) {
	    $items{$item} = 1;
	}
    }
    @items = sort keys %items;
    $t = "";
    foreach $i ( @items ) {
	$proto = $skin->{"table_head.proto"};
	$proto =~ s/\@TEXT\@/$i/g;
	$t .= $proto;
    }
    $text =~ s/\@TABLE_HEAD\@/$t/g;
    $i = $#items+1;
    $text =~ s/\@TABLE_COLUMNS\@/$i/g;
    my $table_text = "";
    while( @groups ) {
	$group = shift @groups;
	$ref = shift @groups;
	$i = $self->{"group_titles"}->{$group};
	$host = $i;
	if( $rowrefto && defined $$rowrefto{$group} ) {
	    $i = $self->table_href( $skin, $$rowrefto{$group}, $i );
	}
	elsif( $refto ) {
	    $i = $self->table_href( $skin, $refto.$group, $i );
	}
	@line = ( $i );
	foreach $i (@items) {
	    if( defined $$ref{$i} ) {
		$item = $$ref{$i};
		$min = $item if( $item<$min );
		$button = $skin->{"table_button.proto"};
		$button =~ s/\@STATUS\@/$state_text[$item]/g;
		if( $itemref ) {
		    my( $ref );

		    $ref = $itemref."/$group.$i";
		    if( -f "$main::root/www/$ref.html" ) {
			$ref .= ".html";
		    }
		    $button = $self->table_href( $skin, $ref, $button );
		}
		push( @line, $button );
	    }
	    else {
		push( @line, "-" );
	    }
	}
	$t = shift @line;
	$proto = $skin->{"table_titlecolumn.proto"};
	$proto =~ s/\@TEXT\@/$t/g;
	$t = $proto;
	foreach $i ( @line ) {
	    $proto = $skin->{"table_cell.proto"};
	    $proto =~ s/\@TEXT\@/$i/g;
	    $t .= $proto;
	}
	$proto = $skin->{"table_line.proto"};
	$proto =~ s/\@TEXT\@/$t/g;
	$lines{$host}=$proto;
    }
    foreach $i ( sort keys %lines ) {
	$table_text .= $lines{$i};
    }
    $text =~ s/\@TABLE_BODY\@/$table_text/g;
    $$vars{"MIN"} = $min;
    return $text."\n";
}


sub table_href {
    my( $self, $skin, $href, $text ) = @_;

    my $proto = $skin->{"table_href.proto"};
    $proto =~ s/\@HREF\@/$href/g;
    $proto =~ s/\@TEXT\@/$text/g;
    return $proto;
}


sub clear {
    my( $self ) = @_;
    my( $images, $image );

    print STDERR "bbdisp::clear\n" if( $main'dl>1 );
    $self->{"isdirty"} = 0;
    if( time>$self->{"synctime"} ) {
	$self->{"synctime"} = time+4*60;
    }
    if( time>$self->{"grouptime"} && $self->{"groupdirty"} ) {
	$self->{"grouptime"} = time+15*60;
	$self->{"groupdirty"} = 0;
    }
    $images = $self->{"images"};
    foreach $image (@$images) {
	$image->updated;
    }
}



sub get_skin {
    my( $self, $skin, $what ) = @_;

    return undef unless( defined $skin->{$what} );
    if( $skin->{"T.".$what} ne "url" ) {
	return( $skin->{$what} );
    }
    my $text = $skin->{$what};
    if( (substr( $text, 0, 1 ) ne "/") && (defined $skin->{"_BASE"}) ) {
	return( $skin->{"_BASE"}."/".$text );
    }
    return $text;
}


sub find_skin {
    my( $self, $name ) = @_;
    my( $dir, $fname );

    print STDERR "find_skin( $name )\n" if( $main::dl>5 );
    foreach $dir ( $main::root."etc/skins/", $main::root."/www/skins/" ) {
	$fname = $dir.$name."/" if( -d $dir.$name );
    }
    return( $fname );
}


sub bypass_skincache {
    my( $self, $bypass ) = @_;
    $self->{"bypass_skincache"} = ! $bypass;
}


sub read_skin {
    my( $self, $name, $skin ) = @_;
    my( $fname, $file );

    print STDERR "read_skin( $name )\n" if( $main::dl>2 );
    if( ($name eq "default") && (! $skin) && (defined $self->{"defaultskin"}) ) {
	$skin = $self->{"defaultskin"};
	my %skin = %$skin;
	return( \%skin );
    }
    $skin = {} unless( $skin );
    $fname = $self->find_skin( $name );
    unless( $fname ) {
	print STDERR "$main'prog: WARNING: skin $name unknown (line $.)"; 
	return $skin;
    }

    if( (-s "$fname/cache") && (! $self->{"bypass_skincache"}) ) {
	print "reading skin from cache\n" if( $main::dl>4 );
	open( _CACHE, "<$fname/cache" );
	while( <_CACHE> ) {
	    chomp;
	    my( $var, $val ) = split( "\x00", $_, 2 );
	    $val =~ tr/\x00/\n/;
	    $skin->{$var} = $val;
	}
	close _CACHE;
	return $skin;
    }


    opendir( _DIR, $fname ) ||
	print STDERR "$main'prog: WARNING: cannot read directory $fname in line $. of config\n"; 
    foreach $file (readdir( _DIR )) {
	if( $file =~ /\.(proto|inc|sub)$/ ) {
	    $skin->{$file} = "";
	    open( _F, "<${fname}$file" );
	    while( <_F> ) {
		$skin->{$file} .= $_;
	    }
	    $skin->{$file} =~ s/\\\n$//;
	    close( _F );
	    $skin->{"T.".$file} = "file";
	}
	else {
	    $skin->{$file} = $fname.$file;
	    $skin->{$file} =~ s#^\Q$main::root/www/##;
	    $skin->{"T.".$file} = "url";
	}
    }
    return $skin;
}


sub cache_skin {
    my( $self, $name, $skin ) = @_;
    my( $dir, $fname, $file );
    my( $var, $val );

    print STDERR "cache_skin( $name )\n" if( $main::dl>2 );
    $skin = {} unless( $skin );
    $fname = $self->find_skin( $name );
    unless( $fname ) {
	print STDERR "$main'prog: WARNING: skin $name unknown"; 
	return undef;
    }
    open( OUT, ">$fname/cache" );
    while( ($var,$val) = each %$skin ) {
	$val =~ tr/\n/\x00/;
	print OUT $var."\x00".$val."\n";
    }
    close OUT;
}



sub encode {
    my( $self, $text ) = @_;
    my $trans;

    $trans = "";
    while( $text =~ /([^a-z0-9_\-])/ ) {
	$trans .= $`."%".sprintf("%x",ord($1));
	$text = $';
    }
    return $trans.$text;
}


1;
