#    Big Sister network monitor
#    Copyright (C) 1998  Thomas Aeby
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

use Platforms;
use strict;
use Getopt::Long;

sub proginit {
    $main::log_facility = (Platforms::iswin32)?"Srv":"daemon";
    $main::progdir = $0;
    $main::prog = $0;
    $main::prog =~ s#.*[/\\]##;
    $main::progdir =~ s#/[^\/\\]*$##;
    my $bs_instdir="/usr/local/lib/bs";

    if( Platforms::isunix() ) {
	eval {
	    require Sys::Syslog;
	    $main::syslog = 1;
	};
    }
    if( Platforms::iswin32() ) {
	eval {
	    require Win32::EventLog;
	    $main::eventlog = 1;
	};
    }
     
    $main::root = Platform::override_root();
    $main::root = $bs_instdir unless( $main::root );
    $Getopt::Long::ignorecase = 0;  
    GetOptions( "D:i", "b:s", "L:s", @main::options ) || &usage;
    $main::dl = $main::opt_D;
    if( $main::opt_b ) {
	$main::root = $main::opt_b;
    }
    load_resources();
    if( $main::resources{"log_facility"} ) {
	$main::log_facility = $main::resources{"log_facility"};
    }
    if( $main::opt_L ) {
	$main::log_facility = $main::opt_L;
    }

    unshift( @INC, $main::root );
    if( $main::log_facility && ! $main::syslog && ! $main::eventlog ) {
	print STDERR "$main::prog: WARNING: syslog not available - falling back to logger\n" if( $main::dl );
    }

    Sys::Syslog::openlog( $main::prog.":", "cons,nowait", $main::log_facility ) if( $main::syslog );
    $main::eventlog = Win32::EventLog->new( $main::log_facility ) if( $main::eventlog );
    &log( "debug", "starting up" );
    $main::hostname = Platform::hostname();
}



sub usage {
    print "Usage: ".($main::prog)." ".($main::Usage)."\n";
    exit 1;
}


sub background {
    &log( "info", "entering daemon mode" );
    if( Platforms::isunix() ) {
	chdir "/";
	close STDIN; close STDOUT; close STDERR;
	if (open(DEVTTY, "/dev/tty")) {
	    ioctl(DEVTTY,0x20007471,0);         # XXX s/b &TIOCNOTTY
	    close DEVTTY;
	}
	open(STDIN,"</dev/null");
	open(STDOUT,">/dev/null");
	open(STDERR,">&STDOUT");
	fork && exit;
	$SIG{"HUP"} = $SIG{"ALRM"} = $SIG{"PIPE"} = $SIG{"INT"} = "IGNORE";
	setpriority( "PRIO_PROCESS", 0, 10 );
    }
    my $pidfile = $main::prog;
    $pidfile =~ s/\.exe$//i;
    open( PID, ">$main::root/adm/$pidfile.pid" );
    print PID "$$\n";
    close PID;
}


sub load_resources {
    my $file;
    %main::resources = ();
    foreach $file( "/etc/resources", "/adm/resources" ) {
	if( -f $main::root.$file ) {
	    open( RES, "<$main::root$file" );
	    while( <RES> ) {
		chomp;
		if( /^($main::prog|\*)\.([a-zA-Z0-9]+)[\s\t]*=[\s\t]*(.*)/ ) {
		    $main::resources{$2} = $3;
		}
	    }
	    close RES;
	}
    }
}


sub log {
    my( $severity, @messages ) = @_;
    my( $message, $level );

    foreach $message (@messages) {
	$level = 3;
	($severity =~ /(alert|err|crit)/) && ($level = 1);
	($severity =~ /(warning|notice)/) && ($level = 2);
	($severity =~ /(info)/) && ($level = 3);
	if( ($main::dl && ($severity =~ /(alert|err|crit)/))
	    || ( ($main::dl>1) && ($severity =~ /(warning|notice)/) )
	    || ( ($main::dl>2) && ($severity =~ /(info)/) )
	    || ($main::dl>3) ) {
	    print STDERR (scalar localtime).": ".$main::prog.": ".$message."\n";
	}
	next unless( $main::log_facility );
	if( $main::syslog ) {
	    Sys::Syslog::syslog( $severity, $message );
	}
	elsif( $main::eventlog ) {
	    my %event = ('EventID' => 100, 
		'EventType' => $level, 
		'Strings' => $main::prog.": ".$message, 
		"Source" => "Perl" );

	    $main::eventlog->Report( \%event );
	}
	else {
	    $severity =~ s/\|.*//;
	    system( "logger", "-p", $main::log_facility.".".$severity, "-i", $main::prog.":", $message );
	}
    }
}

	    
	
1;
