#!/usr/bin/perl -I/usr/local/lib/bs/bin
#    Big Sister network monitor
#    Copyright (C) 1998  Thomas Aeby
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#=============================================================================
#
my $Usage	  = "[-D level] [-l logdir] [-b bigsisterdirectory] [-c configfile] [-h historyfile]";
#
#=============================================================================

# Trim the program path name
$prog = $progdir = $0;
$prog =~ s#.*/##;
$progdir =~ s#/[^\/]*$##;
$progdir =~ /^(.*)$/; $progdir = $1; # untaint ...
 
use Getopt::Long;
use strict;
use Socket;
use FileHandle;

my %status_values = (
    "red", 0,
    "green", 3,
    "yellow", 1,
    "purple", 2
    );

BEGIN { $ENV{PATH} = '/usr/ucb:/bin:/usr/bin:/sbin:/usr/sbin' }

$SIG{"ALRM"} = $SIG{"PIPE"} = "IGNORE";

$Getopt::Long::ignorecase = 0;  
$main::opt_D = 0;
GetOptions( "D:i", "l:s", "b:s", "c:s", "h:s" ) || &usage;
my $dl = $main'dl = $main::opt_D;

if( $main::opt_b ) {
    # untaint ...
    $main::opt_b =~ /^(.*)$/;
    $main::root = $1;
}
elsif( -d "/l/lib/bs" ) {
    $main::root = "/l/lib/bs";
}
elsif( -d "/usr/local/lib/bs" ) {
    $main::root = "/usr/local/lib/bs";
}
else {
    $main::root = $main::progdir."/..";
}

my $logdir = "$main::root/www/logs";
$logdir = $main::opt_l if( $main::opt_l );

unshift( @INC, "$main::root/bin" );

my $history_file = "$main::root/www/logs/history/display.history";
$history_file = $main::opt_h if( $main::opt_h );
my $config_file = "$main::root/adm/bs_event_generator.cfg";
$config_file = $main::opt_c if( $main::opt_c );

&background unless( $main::dl );
%status = ();
&open_hist;
&read_state( $logdir );


sub open_hist {
    open( HIST, "<$main::history_file" );
    lseek( HIST, 0, 2 );
    $main::history_pos = tell( HIST );
    $main::history_inode = ((stat($history_file))[1]);
}


sub read_state {
    my( $logdir, $file ) = @_;
    my( $host, $item );

    opendir( DIR, $logdir ) || return;
    foreach $file (readdir(DIR)) {
	if( (-f $logdir."/".$file) && ($file =~ /^([^\.]+)\.([^\.]+)$/) ) {
	    ($host, $item) = ($1, $2);
	    open( IN, "<$logdir/$file" );
	    $_ = <IN>;
	    close IN;
	    chomp;
	    /^(red|green|purple|yellow)/ && ($status{"$host.$item"} = $status_values{$1});
	}
    }
}



sub background {
    chdir "/";
    close STDIN; close STDOUT; close STDERR;
    if (open(DEVTTY, "/dev/tty")) {
	ioctl(DEVTTY,0x20007471,0);         # XXX s/b &TIOCNOTTY
	close DEVTTY;
    }
    open(STDIN,"<&TTY");
    open(STDOUT,">&TTY");
    open(STDERR,">&STDOUT");
    fork && exit;
    setpriority( "PRIO_PROCESS", 0, 10 );
}
	


