package Monitor::bb;

require 5.002;
require Exporter;

use strict;
use vars qw(@ISA @EXPORT $VERSION);
use Socket qw( SOCK_STREAM PF_INET inet_aton sockaddr_in );
use FileHandle;
use Carp;

@ISA = qw(Exporter Monitor::Monitor);


sub new
{
    my ($this, $host, $timeout, $port) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    print "Monitor::bb new()\n" if( $main::dl>1 );
    bless($self, $class);

    ($timeout = 8) if( ! $timeout );
    $self->{"timeout"} = $timeout;

    ($port = "bb") if( ! $port );
    unless( $port =~ /^\d+$/ ) { $port = (getservbyname($port, 'tcp'))[2]; }
    ($port = 1984) if( ! $port );

    $self->{"port"} = $port;
    $self->{"host"} = $host;
    $self->{"proto_num"} = (getprotobyname('tcp'))[2];
    $self->{"fh"} = FileHandle->new();
    $self->{"FQDN_sep"} = "_";
    $self->{"FQDN"} = 1;

    return($self);
}


sub fqdn {
    my( $self, $fqdn ) = @_;

    $self->{"FQDN"} = $fqdn;
}


sub bb_compat {
    my( $self, $compat ) = @_;

    $self->{"compat"} = $compat;
    $self->{"FQDN_sep"} = ($compat?",":"_");
}


sub set_time {
    my( $self, $time ) = @_;

    $self->{"override_time"} = $time;
}


sub run {
    my( $self, @args ) = @_;
    $self->report( @args );
}


sub report
{
    my ($self, $status, $comment, $join) = @_;
    my( $date, $item, $item_status, $ip, $shitem, $add );

    $ip = inet_aton( $self->{"host"} );
    unless( $ip ) {
	print "to bb: error, host unknown: ".$self->{"host"}."\n" if( $main::dl>1 );
	return;
    }
    $self->_start_report( $ip ) unless( $self->{"compat"} );
    $date = time unless( $date = $self->{"override_time"} );
    while( ($item, $item_status) = each( %$join ) ) {
	next unless( @$item_status );
	$shitem = substr( $item, 1 );
	$add = substr( $item, 0, 1 );
	($add eq "+") && ($add = "join");
	($add eq "-") && ($add = "leave");
	($add eq "D") && ($add = "displayname");
	$self->report_hw( "$add $shitem ".join(" ",@$item_status) ) unless( $self->{"compat"} );
    }
    while( ($item,$item_status) = each( %$status ) ) {
	print "to bb: $item: $item_status (".$$comment{$item}.")\n" if( $main::dl>3 );
	$self->_start_report( $ip ) if( $self->{"compat"} );
	$self->report_one( $ip, $item, $item_status, $$comment{$item}, $date ); 
	$self->_end_report( $ip ) if( $self->{"compat"} );
    }
    $self->_end_report unless( $self->{"compat"} );
}



sub sysreport {
    my( $self, $msg ) = @_;

    unless( $self->{"connected"} ) {
	my $ip = inet_aton( $self->{"host"} );
	unless( $ip ) {
	    print "to bb: error, host unknown: ".$self->{"host"}."\n" if( $main::dl>1 );
	    return;
	}
	$self->_start_report( $ip );
    }
    return unless( $self->{"connected"} );
    $self->report_hw( $msg );
    if( $self->{"compat"} ) {
	$self->_end_report();
    }
}



sub _start_report
{
    my ($self, $ip) = @_;
    my ($timeout, $saddr );
    my( $port, $res );

    $timeout = $self->{"timeout"};
    $port = $self->{"port"};

    socket($self->{"fh"}, &PF_INET(), &SOCK_STREAM(), $self->{"proto_num"}) || return( 0 );
    $saddr = sockaddr_in($port, $ip);

    $res = $self->{"connected"} = 0;
    eval {
	$SIG{'ALRM'} = sub { die "Monitor::bb:_start_report timed out" };
	Platform::bsalarm($timeout); 
	    
	if( Platform::bsconnect($self->{"fh"}, $saddr) ) {
	    $self->{"connected"} = $res = 1;
	}
	Platform::bsalarm(0);
    };
    Platform::bsalarm(0);
    close $self->{"fh"} unless( $res );
    return $res;
}



sub _end_report {
    my( $self ) = @_;

    select( undef, undef, undef, 0.5 );
    close( $self->{"fh"} );
    select( undef, undef, undef, 0.1 );
    $self->{"connected"} = 0;
}



sub report_one
{
    my ($self, $ip, $item, $status, $comment, $date) = @_;

    if( $self->{"FQDN"} ) {
        $item =~ s/[\._](.*)\./\./;
    }
    else {
        my @tok = split( /[\._]/, $item );
        my $check = pop @tok;
        $item = join( $self->{"FQDN_sep"}, @tok ).".".$check;
    }

    if( $self->{"compat"} ) {
	$comment =~ s/\|\>/\n/g;
    }
    else {
	$comment =~ s/\n/\|\>/g;
    }
    return( $self->report_hw( "status $item ".(("red","yellow","purple","green")[$status])." ($date) ".localtime($date)." $comment" ) );
}


sub report_hw
{
    my ($self, $msg ) = @_;
    my ($timeout, $saddr );
    my( $res );

    $timeout = $self->{"timeout"};
    return(0) unless( $self->{"connected"} );

    $res = 0;
    eval {
	$SIG{'ALRM'} = sub { die };
	Platform::bsalarm($timeout); 
	    
	$msg .= "\n";
	print "reporter reporting: ".$msg if( $main::dl>5 );
	unless( send( $self->{"fh"}, $msg, 0 ) ) {
	    $res = 0;
	    $self->_end_report;
	}
	Platform::bsalarm(0);
	$res = 1;
    };
    Platform::bsalarm(0);
    return $res;
}


sub socket {
    my( $self ) = @_;

    return( $self->{"fh"} );
}

1;
