package Monitor::cpuload;

require 5.002;

#use strict;

@Monitor::cpuload::ISA = qw(Monitor::Monitor);

# create the objects + memory space needed
sub new
{

    my ($this, $pscomm) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);

    # create an array to store the CPU load limits info
    $self->{"htable"} = [];

    return($self);
}


sub add_check 
{
    my( $self, $max1, $max2, $item ) = @_;
    my( $htable ) = $self->{"htable"};

    # figure out how to report this data ( server name + monitor name )
    ($item = $uxmon::hostname.".cpu") unless( $item );

    # report on what's going on - if we're in debug mode
    if( $main::dl>3 )
    {
        print "add_check - item: '$item'\n";
    }

    # add this info to the configuration array
    push( @$htable, join( "\x00", $item, $max1, $max2 ) );
}


sub check
{
    my ($self) = @_;

    my( $htable ) = $self->{"htable"};

    # get the cpu load averages using 'uptime'
    my $uptime = `uptime`;

    # report on what's going on - if we're in debug mode
    if( $main::dl>3 )
    {
         print "uptime: '$uptime'\n";
    }

    # retrieve the data from the info returned by uptime
    $uptime =~ /load average: (.+)$/;
    my $loads = $1;

    # report on what's going on - if we're in debug mode
    if( $main::dl>3 )
    { 
        print "load: '$loads'\n";
    }

    # now split the data into it's three numeric components
    my ($load1,$load2, $load3);

    ($load1,$load2, $load3) = split (',',$loads);

    # report on what's going on - if we're in debug mode
    if( $main::dl>3 )
    {
        print "load1: '$load1'\n";
        print "load2: '$load2'\n";
        print "load3: '$load3'\n";
    }

    # calculate the avarage of these CPU load results
    my $ave = ($load1 + $load2 + $load3 ) /3 ;

    # loop throught every item to be checked (probably only one per box)
    for( $i=0; $i<=$#$htable; $i++ ) 
    {
        ( $item, $max1, $max2 ) = split( "\x00", $$htable[$i] );

        # report on what's going on - if we're in debug mode
        if( $main::dl>3 )
        {
            print "working on '$i' : item: '$item' max1:'$max1' max2:'$max2'\n";
        }

        #  check for errors
        if (! length($ave) )
        {
            # report to main monitoring server the status
            $self->report( $item, 1, "CPU load average - no data from 'uptime'" );
               
            # report on what's going on - if we're in debug mode
            if( $main::dl>3 )
            {
                print "CPUload error! (couldn't get data from
'uptime')\n";
            }
        }
        else
        {
            # see how to report on CPU load - red, yellow or GREEN
            if( (length($max2) && ($ave>$max2)) ) {
                $self->report( $item, 0, "CPU load >>($ave)" );
            }
            elsif( (length($max1) && ($ave>$max1)) ) {
                $self->report( $item, 1, "CPU load >($ave)" );
            }
            else {
                $self->report( $item, 3, "CPU load = $ave" );
            }
            
            # report on what's going on - if we're in debug mode
            if( $main::dl>3 )
            {
                print "Made it through cpuload.pm::check just fine - ";
                print "CPUload worked!\n";
            }
        }
    }
}

1;
