package Monitor::fs;

require 5.002;

use strict;

@Monitor::fs::ISA = qw(Monitor::Monitor);


sub new
{
    my ($this, $fstype, $fscomm) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);

    unless( $fscomm ) {
	$fscomm = "df";
	system( "$fscomm -k -F ufs > /dev/null 2>&1" );
	if( $? ) {
	    # BSD
	    ($fstype = "4.2") unless( $fstype );
	    $fscomm .= " -t $fstype";
	 }
	 else {
	    # Solaris 2
	    ($fstype = "ufs") unless( $fstype );
	    $fscomm .= " -k -F $fstype";
	 }
    }

    $self->{"fscomm"} = $fscomm;
    my( @bla)  = ();
    $self->{"htable"} = \@bla;

    return($self);
}


sub add_check {
    my( $self, $mp, $item, $yellowlevel, $redlevel ) = @_;
    my( $htable ) = $self->{"htable"};

    ($item = $uxmon::hostname.".procs") unless( $item );
    push( @$htable, join( "\x00", $mp, $item, $yellowlevel, $redlevel ) );
}





sub check
{
    my ($self) = @_;
    my( $htable ) = $self->{"htable"};
    my( $item, $yellowlevel, $redlevel );
    my( $i, $n );
    my( %df );
    my( $dev, $total, $used, $free, $usage, $mp, $stat, $txt, $ntxt, $nstat );

    return unless( @$htable );
    open( _PROC,  $self->{"fscomm"}."|" );
    while( <_PROC> ) {
	chomp;
	next if( /^File/ );
	( $dev, $total, $used, $free, $usage, $mp ) = split( /[\s\t]+/ );
	$df{$mp} = $free;
    }
    close _PROC;

    for( $i=0; $i<=$#$htable; $i++ ) {
	($mp, $item, $yellowlevel, $redlevel) = split( "\x00", $$htable[$i] );
	$n = $df{$mp};
	if( $n<$redlevel ) {
	    $nstat = 0;
	    $ntxt = "$mp ${n}k (<<) free";
	}
	elsif( $n<$yellowlevel ) {
	    $nstat = 1;
	    $ntxt = "$mp ${n}k (<) free";
	}
	else {
	    $nstat = 3;
	    $ntxt = "$mp ${n}k free";
	}
	$self->report( $item, $nstat, $ntxt );
    }
}

1;
