package Monitor::procs;

require 5.002;

use strict;

@Monitor::procs::ISA = qw(Monitor::Monitor);

sub new
{
    my ($this, $pscomm) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);

    unless( $pscomm ) {
	$pscomm = "/usr/ucb/ps";
	(-x $pscomm) || ($pscomm = "/bin/ps");
	(-x $pscomm) || ($pscomm = "/usr/bin/ps");
	system( "$pscomm cax > /dev/null 2>&1" );
	if( $? ) {
	    # System V
	    $pscomm .= " -Afo comm";
	 }
	 else {
	    # BSD - preferred
	    $pscomm .= " cax";
	 }
    }

    $self->{"pscomm"} = $pscomm;
    my( @bla)  = ();
    $self->{"htable"} = \@bla;
    $self->{"domains"} = [];

    return($self);
}


sub add_check {
    my( $self, $comm, $item, $min, $max, $domain ) = @_;
    my( $htable ) = $self->{"htable"};
    my $domains = $self->{"domains"};

    ($domain = ":procs") unless( $domain );
    ($item = $uxmon::hostname.".procs") unless( $item );
    push( @$htable, join( "\x00", $comm, $item, $min, $max, $domain ) );
    push( @$domains, $domain ) unless( grep( $domain eq $_, @$domains ) );
}





sub check
{
    my ($self) = @_;
    my( $index, @table );
    my $domains = $self->{"domains"};
    my $comm;
    my $domain;

    foreach $domain (@$domains) {
	next unless( $domain =~ /^(.*):procs$/ );
	my $host = $1;
	($host = $uxmon::hostname) unless ($host);
	$index = 1;
	if( Platforms::iswin32() ) {
	    require Win32::Service;

	    my $list = {};
	    Win32::Service::GetServices($host, $list);
	    my( $long, $short );
	    while( ($long,$short) = each %$list ) {
		print "Win32 service on $host: $short ($long)\n" if( $main::dl>7 );
		my $status = {};
		Win32::Service::GetStatus( $host, $short, $status );
		if( $status->{"CurrentState"} & 4 ) {
		    $self->set_variable( $host, "procs.hrSWRunName.".$index, $short );
		    push( @table, "procs.hrSWRunName.$index" );
		    $index++;
		    if( $short ne $long ) {
			$self->set_variable( $host, "procs.hrSWRunName.".$index, $long );
			push( @table, "procs.hrSWRunName.$index" );
			$index++;
		    }
		}
	    }
	}
	else {
	    open( _PROC,  $self->{"pscomm"}."|" );
	    while( <_PROC> ) {
		chomp;
		$comm = $_;
		$comm =~ s/^.*[\s]+([^\s]+)[\s]*$/$1/;
		$comm =~ s#.*/##;
		$self->set_variable( $host, "procs.hrSWRunName.".$index, $comm );
		push( @table, "procs.hrSWRunName.$index" );
		$index++;
	    }
	    close _PROC;
	}
	$self->set_variable( $host, "procs.hrSWRunName", \@table );
    }
}


sub late_check
{
    my ($self) = @_;
    my( $htable ) = $self->{"htable"};
    my( $comm, $item, $min, $max );
    my( $i, $n );
    my( %procs );
    my( $table, $domain );
    my $domains = $self->{"domains"};

    foreach $domain (@$domains) {
	print "  procs:late_check(): doing domain $domain\n" if( $main::dl>6 );
	my( $host, $sdomain ) = split( ":", $domain );
	( $host = $uxmon::hostname ) unless( $host );
	my $vars = $uxmon::monitor_variables{$host};
	next unless( $vars );
	my $table = $vars->{$sdomain.".hrSWRunName"};
	next unless( $table );
	my $domprocs = $procs{$domain} = {};
	my $i;
	foreach $i (@$table) {
	    $comm = $vars->{$i};
	    print "  procs:late_check(): adding process $comm ($i)\n" if( $main::dl>7 );
	    $comm =~ s/ .*// if( $sdomain =~ /snmp/ );
	    if( defined $domprocs->{$comm} ) {
		$domprocs->{$comm}++;
	    }
	    else {
		$domprocs->{$comm} = 1;
	    }
	}
    }
	    

    for( $i=0; $i<=$#$htable; $i++ ) {
	($comm, $item, $min, $max, $domain) = split( "\x00", $$htable[$i] );
	print "checking $comm:$item:$min:$max\n" if( $main::dl>4 );
	$n = $procs{$domain}->{$comm};
	($n = 0) unless( defined $n );
	if( $min && ($min>$n) ) {
	    $self->report( $item, 0, "$comm <($n)" );
	}
	elsif( length($max) && ($max<$n) ) {
	    $self->report( $item, 0, "$comm >($n)" );
	}
	else {
	    $self->report( $item, 3, "$comm OK($n)" );
	}
    }
}

1;
