# The snmp Monitor does require Simon Leinen's snmp modules
# They are available at
#	http://www.switch.ch/misc/leinen/snmp/perl/index.html

package Monitor::snmp;

require 5.002;

use strict;
use SNMP_util;

@Monitor::snmp::ISA = qw(Monitor::Monitor);

#$Monitor::snmp::offlmib = "/usr/local/lib/bs/adm/snmptest";

&load_mib( undef, $main::root."/etc/mibs.txt" );

sub new
{
    my ($this, $host, $community ) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);

    unless( $community ) {
	$community = "public";
    }
    $self->{"host"} = $host;
    $self->{"community"} = $community;
    $self->{"lastpoll"} = 0;
    $self->{"freq"} = 4*60;

    $self->{"htable"} = [];
    $self->{"status"} = [];
    $self->{"cache"} = [];
    $self->{"walks"} = [];

    return($self);
}


sub load_mib {
    my( $self, $file ) = @_;

    open( IN, "<$file" );
    while( <IN> ) {
	chomp;
	next if( /^#/ );
	if( /^([^\s\t]+)[\s\t]+([^\s\t]+)$/ ) {
	    $SNMP_util::OIDS{$1} = $2;
	}
    }
    close IN;
}


sub poll_fail {
    my( $self, $item, $text, $state ) = @_;

    $state = 0 unless( defined $state );
    $text = "snmp" unless( defined $text );
    if( $item ) {
	$self->{"poll_fail"} = [ $item, $text, $state ];
    }
    else {
	$self->{"poll_fail"} = undef;
    }
}




sub frequency {
    my( $self, $freq ) = @_;
    $self->{"freq"} = $freq;
    $self->{"freq"} = 4*60 unless( $self->{"freq"} );
}


sub get {
    my( $self, @oid ) = @_;

    if( $Monitor::snmp::offlmib ) {
	 my @ret = ();
	 print "snmpget/debug $Monitor::snmp::offlmib\n" if( $main::dl>3 );
	 my $i;
	 foreach $i (@oid) {
	     push( @ret, undef );
	     open( _IN, "<$Monitor::snmp::offlmib" );
	     while( <_IN> ) {
		 chomp;
		 if( /^$i = (.*)/ ) {
		     close(_IN);
		     print "  snmpget/debug $i:$1\n" if( $main::dl>4 );
		     $ret[$#ret] = $1;
		     last;
		 }
	     }
	     close( _IN );
	 }
	 return( @ret );
    }
    return( snmpget( $self->{"community"}."\@".$self->{"host"}, @oid ) );
}


sub walk {
    my( $self, $oid ) = @_;

    if( $Monitor::snmp::offlmib ) {
	 my( @ret );
	 print "snmpwalk/debug $Monitor::snmp::offlmib\n" if( $main::dl>3 );
	 open( _IN, "<$Monitor::snmp::offlmib" );
	 while( <_IN> ) {
	     chomp;
	     if( /^$oid\.([^\s]+) = (.*)/ ) {
		 push( @ret, "$1:$2" );
		 print "  snmpwalk/debug $1:$2\n" if( $main::dl>4 );
	     }
	 }
	 close( _IN );
	 return( @ret );
    }
    return( snmpwalk( $self->{"community"}."\@".$self->{"host"}, $oid ) );
}


sub add_walk {
    my( $self, $oid ) = @_;
    my( $walks ) = $self->{"walks"};

    push( @$walks, [ $oid ] );
}


sub add_check {
    my( $self, $oid, $item, $name, $differential, $min1, $min2, $max1, $max2 ) = @_;
    my( $htable ) = $self->{"htable"};

    ($item = $self->{"host"}.".snmp") unless( $item );
    push( @$htable, [ $oid, $item, $name, $differential, $min1, $min2, $max1, $max2 ] );
}



sub check
{
    my ($self) = @_;
    my( $htable ) = $self->{"htable"};
    my( @oids, $check, $diff, $i, @noids );
    my( $now ) = time;
    my( @newval, @chkval, $oldstatus, $timediff );

    if( ($timediff = $now - $self->{"lastpoll"}) >= $self->{"freq"} ) {
	$self->{"lastpoll"} = $now;
	$self->{"pollok"} = 0;
	$i = 0;
	@newval = ();
	@noids = ();
	foreach $check (@$htable) {
	    print "    snmp: getting oid $$check[0]\n" if( $main::dl>6 );
	    push( @oids, $$check[0] );
	    push( @noids, $$check[0] );
	    $diff = 1 if( $$check[3] );
	    if( $i++ > 20 ) {
		push( @newval, $self->get( @noids ) );
		$i = 0;
		@noids = ();
	    }
	}
	push( @newval, $self->get( @noids ) ) if( @noids );
	@chkval = @newval;
	if( grep( defined $_, @chkval ) ) {
	    $self->{"pollok"} = 1;
	}
	$oldstatus = $self->{"status"};
	if( $diff ) {
	    for( $i=$#chkval; $i>=0; $i-- ) {
		$check = $$htable[$i];
		if( $$check[3] ) {
		    if( defined $$oldstatus[$i] ) {
			$chkval[$i] -= $$oldstatus[$i];
			if( $$check[3] eq "PERSEC" ) {
			    $chkval[$i] /= $timediff;
			}
		    }
		    else {
			$chkval[$i] = undef;
		    }
		}
	    }
	 }

	 my $walks = $self->{"walks"};
	 foreach $check (@$walks) {
	     my( $oid ) = @$check;
	     my( @subs ) = $self->walk( $oid );
	     my $i;
	     print "    snmp: doing snmpwalk for $oid\n" if( $main::dl>5 );
	     foreach $i (@subs) {
		 print "      snmp: sub = $i\n" if( $main::dl>6 );
		 my $pos = index( $i, ":" );
		 next if( $pos<0 );
		 my $soid = substr( $i, 0, $pos );
		 my $val = substr( $i, $pos+1 );
		 $val =~ s/^\"//;
		 $val =~ s/\".*//;
		 $i = "snmp.".$oid.".".$soid;
		$self->set_variable( $self->{"host"}, "snmp.".$oid.".".$soid, $val );
	    }
	    $self->set_variable( $self->{"host"}, "snmp.".$oid, \@subs );
	}

	undef $self->{"cache"};
	$self->{"cache"} = \@chkval;
	undef $self->{"status"};
	$self->{"status"} = \@newval;
    }

    if( $self->{"poll_fail"} ) {
	my $pf = $self->{"poll_fail"};

	if( $self->{"pollok"} ) {
	    $self->report( $$pf[0], 3, $$pf[1]." OK" );
	}
	else {
	    $self->report( $$pf[0], $$pf[2], $$pf[1]." FAILURE" );
	}
    }
	    
    my( $cache ) = $self->{"cache"};
    my( $status ) = $self->{"status"};
    my( $val );
    my( $oid, $item, $name, $differential, $min1, $min2, $max1, $max2 );
    for( $i=0; $i<=$#$htable; $i++ ) {
	$check = $$htable[$i];
	( $oid, $item, $name, $differential, $min1, $min2, $max1, $max2 ) = @$check;
	if( $main::dl>3 ) {
	    print "snmp:$oid, $item, $name -> status: ".$$status[$i]." check: ".$$cache[$i]."\n";
	}
	unless( defined $$cache[$i] ) {
	    $self->report( $$check[1], 3, $$check[2]. " (undefined)" );
	    $self->set_variable( $self->{"host"}, "snmp.".$oid, undef );
	}
	else {
	    $val = $$cache[$i];
	    $self->set_variable( $self->{"host"}, "snmp.".$oid, $val );
	    if( defined $min1 || defined $min2 || defined $max1 || defined $max2 ) {
		if( (length($min1) && ($val<$min1)) ) {
		    $self->report( $item, 0, "$name <<($val)" );
		}
		elsif( (length($min2) && ($val<$min2)) ) {
		    $self->report( $item, 1, "$name <($val)" );
		}
		elsif( (length($max2) && ($val>$max2)) ) {
		    $self->report( $item, 0, "$name >>($val)" );
		}
		elsif( (length($max1) && ($val>$max1)) ) {
		    $self->report( $item, 1, "$name >($val)" );
		}
		else {
		    $self->report( $item, 3, "$name = $val" );
		}
	    }
	}
    }
}

1;
