package Monitor::tcp;

require 5.002;
require Exporter;

use strict;
use vars qw(@ISA @EXPORT $VERSION);
use Socket qw( SOCK_STREAM PF_INET inet_aton sockaddr_in );
use FileHandle;
use Carp;

@Monitor::tcp::ISA = qw(Exporter Monitor::Monitor);

%Monitor::tcp::known_ports = (
    "pop3", 110,
    "smtp", 25,
    "ica", 1494,
    "nntp", 119,
    "smb", 139,
    "dns", 53,
    "ssh", 22,
    "telnet", 23,
    "time", 37,
    "bootps", 67,
    "gopher", 70,
    "http", 80,
    "kerberos", 88,
    "sunrpc", 111,
    "auth", 113,
    "ntp", 123,
    "netbios-ns", 137,
    "netbios-dgm", 138,
    "netbios-ssn", 139,
    "ftp", 21
);


sub new
{
    my ($this, $timeout) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);

    $self->{"timeout"} = $timeout;

    $self->{"proto_num"} = (getprotobyname('tcp'))[2];
    $self->{"fh"} = FileHandle->new();
    $self->{"settle"} = 2;

    my( @bla)  = ();
    $self->{"htable"} = \@bla;

    return($self);
}


sub add_hosts {
    my( $self ) = shift;
    my( $htable ) = $self->{"htable"};

    push( @$htable, @_ );
}


sub add_any {
    my( $self, $host, $port, $item, $spec1, $spec2 ) = @_;

    ($port eq "nntp") && return $self->add_nntp( $host, $item );
    ($port eq "pop3") && return $self->add_pop3( $host, $item );
    ($port eq "smtp") && return $self->add_smtp( $host, $item );
    ($port eq "http") && return $self->add_http( $host, $item, $spec1, $spec2 );
    ($port eq "telnet") && return $self->add_telnet( $host, $item );
    ($item = "$host.$port") unless( $item );
    $self->add_hosts( "$host\x00$port", $item );
}


sub add_nntp {
    my( $self, $host, $item ) = @_;

    ($item = "$host.nntp") unless defined $item;
    $self->add_hosts( $host."\x00nntp\x00^20", $item );
}


sub add_ica {
    my( $self, $host, $item ) = @_;
    my( $port );

    $port = "ica";
    ($item = "$host.ica") unless defined $item;
    $self->add_hosts( $host."\x00$port\x00ICA", $item );
}


sub add_pop3 {
    my( $self, $host, $item ) = @_;
    my( $port );

    $port = "pop3";
    ($port = "pop-3") if( getservbyname( "pop-3", "tcp" ) );
    ($item = "$host.pop3") unless defined $item;
    $self->add_hosts( $host."\x00$port\x00^\\+OK", $item );
}


sub add_telnet {
    my( $self, $host, $item ) = @_;
    my( $port );

    $port = "telnet";
    ($item = "$host.telnet") unless defined $item;
    $self->add_hosts( $host."\x00$port\x00.", $item );
}


sub add_smtp {
    my( $self, $host, $item ) = @_;
    my( $port );

    $port = "smtp";
    ($item = "$host.smtp") unless defined $item;
    $self->add_hosts( $host."\x00$port\x00^22", $item );
}


sub add_http {
    my( $self, $host, $item, $url, $port ) = @_;

    ($port = "http") unless defined $port;
    ($url = "/") unless defined $url;
    ($item = "$host.http") unless defined $item;
    $self->add_hosts( "$host\x00$port\x00^HTTP.[\\.0-9]+ [24]\x00GET $url HTTP/1.0\n\n", $item );
}



sub check
{
    my ($self) = @_;
    my ($ip, $i, @list, %result, $timeout, $res, $rbits, $finish_time, $msg, $saddr, $nfound, $aport );
    my( $htable ) = $self->{"htable"};
    my( $host, $port, $nport, $expect, $send, $item );

    $timeout = $self->{"timeout"};
    @list = @$htable;

    for( $i=0; $i<=$#list; $i+=2 ) {
	$item = $list[$i+1];
	undef $host;
	undef $port;
	undef $expect;
	undef $send;
	($host,$port,$expect,$send) = split( "\x00", $list[$i] );
	print "checking tcp($host,$port,$expect,$send)\n" if( $main::dl>4 );
	$ip = inet_aton($host);
	next unless( $ip );
	$aport = $port;
	$nport = (getservbyname($port, 'tcp'))[2];
  	if( $nport ) { $port = $nport; }
	 else {
	    $nport = $Monitor::tcp::known_ports{$port};
	    if( $nport ) { $port = $nport; }
	}

	socket($self->{"fh"}, &PF_INET(), &SOCK_STREAM(), $self->{"proto_num"}) || next;
	$saddr = sockaddr_in($port, $ip);

	$res = 0;
	$finish_time = time + $timeout;
	eval {
	    $SIG{'ALRM'} = sub { die };
	    Platform::bsalarm($timeout); 
		
	    die unless Platform::bsconnect($self->{"fh"}, $saddr);
	    if( $send ) {
		send( $self->{"fh"}, $send, 0 );
	    }
	    if( $expect ) {
		$timeout = $finish_time - time(); 
		$msg = "";
		$rbits = "";
		vec($rbits, $self->{"fh"}->fileno(), 1) = 1;

		while( ($timeout > 0) && (! $res) && (length($msg)<4096)  ) {
		    $nfound = select($rbits, undef, undef, $timeout); 
		    $timeout = $finish_time - time(); 
		    die unless( defined $nfound );
		    if( $nfound ) {
			recv( $self->{"fh"}, $msg, 1024, 0 );
			if( $msg =~ /$expect/ ) {
			    $res = 3;
			}
		    }
		}
	    }
	    else { 
		Platform::bsalarm(0);
		$res = 3;
		sleep $self->{"settle"};
	    }
	    Platform::bsalarm(0);
	};
	Platform::bsalarm(0);
	$self->{"fh"}->close();
	$self->report( $item, $res, "tcp ".$aport." ".($res?"OK":"FAILURE") );
    }

}

1;
