#    Big Sister network monitor
#    Copyright (C) 1998  Thomas Aeby
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package rules;
use Socket;
use FileHandle;
local( %args, %memory );

$rules::grouping_stat = 0;
$rules::config_stat = 0;

# the procedure rules::config is called each time the file is read
sub config {
    my( $chk );
    my( $config ) = "$main::root/adm";
    my( $host, @attr, $attr, $alias );
    my( $localhost, $fh );

    %memory = ();
    undef $rules::ping;
    %rules::memory = %rules::args = ();

    # read file adm/grouping
    if( -f "$config/grouping" ) {
	my( $name, $realname, $groups );
	$rules::grouping_stat = ((stat("$config/grouping"))[9]);
	open( IN, "<$config/grouping" );
	while( <IN> ) {
	    chomp;
	    next if( /^#/ );
	    next if( /^$/ );
	    $realname = "";
	    if( /\((.*)\)/ ) {
		$realname = $1;
		$_ = $`.$';
	    }
	    ($name,@groups) = split( /[\s\t]+/ );
	    $name =~ s/\./_/g;
	    &uxmon::join_group( $name, @groups ) if( @groups );
	    &uxmon::displayname( $name, $realname ) if( $realname );
	}
	close( IN );
	&uxmon::leave_on_exit(1);
    }
		
    # read config from uxmon-net if present
    $localhost = 0;
    my $cfg = $uxmon::configfile;
    unless( $cfg ) { $cfg = "uxmon-net"; }
    unless( $cfg =~ /\// ) { $cfg = $config."/".$cfg; }
    if( -f $cfg ) {
	$rules::config_stat = ((stat("$cfg"))[9]);
	$fh = FileHandle->new();
	open( $fh, "<$cfg" );
	while( <$fh> ) {
	    chomp;
	    while( /\\$/ ) {
		$chk = $_;
		$_ = <$fh>;
		last unless( $_ );
		chomp;
		$_ = substr($chk,0,length($chk)-1).$_;
	    }
	    s/#.*//;
	    %args = ();
	    ($host, @attr) = &parse( $_ );
	    next unless( $host );
	    $alias = "";
	    if( $host =~ /^(.*)\((.*)\)/ ) {
		$host = $1;
		$alias = $2;
	    }
	    $host = $uxmon::hostname if( $host eq "localhost" );
	    $alias = $host unless( $alias );
	    if( $host eq $uxmon::hostname ) {
		$localhost++;
		$args{"localhost"} = 1;
	    }
	    $rules::args{"host"} = $host;
	    $rules::args{"alias"} = $alias;
	    foreach $attr (@attr) {
		&execute( $attr );
	    }
	}
        close $fh;
    }

    # if no display server was configured use host 'bsdisplay' as a default
    unless( @uxmon::reporters ) {
	$args{"host"} = "bsdisplay";
	&execute( "bsdisplay" );
    }
}


sub execute {
    my( $attr ) = @_;

    $args{"cmd"} = $attr;
    print "uxmon-rules.pl: executing $attr for host $args{host}\n" if( $main::dl>1 );
    unless( inet_aton($args{"host"}) ) {
	print STDERR "$main'prog: WARNING: host $args{'host'} appearing in $cfg is unresolvable\n";
    }
    $attr =~ s/^(http):.*/$1/;
    if( $attr =~ /^(.*?)=(.*)/ ) {
	$args{$1} = $2;
    }
    elsif( -f "$main::root/adm/Config/$attr" ) {
	chdir( "$main::root/adm/Config" );
	do "$attr";
    }
    elsif( -f "$main::root/uxmon/Config/$attr" ) {
	chdir( "$main::root/uxmon/Config" );
	do "$attr";
    }
    else {
	my $ok = 0;
	eval {
	    do "$attr";
	    $ok = 1;
	};
	main::log( "warn", "check $attr unknown - ignored" ) unless( $ok );
    }
}


sub reload {
    my( $config ) = "$main::root/adm";

    my $cfg = $uxmon::configfile;
    unless( $cfg ) { $cfg = "uxmon-net"; }
    unless( $cfg =~ /\// ) { $cfg = $config."/".$cfg; }
    if( -f "$config/grouping" ) {
	return( 1 ) if( ((stat(_))[9]) != $rules::grouping_stat );
    }
    if( -f "$cfg" ) {
	return( 1 ) if( ((stat(_))[9]) != $rules::config_stat );
    }
    return(0);
}



sub parse {
    my( $str ) = @_;
    my @res = ();
    my $buff = "";

    while( $str =~ /^([^\s\t\"\']*)([\s\t]+|[\"\'])/ ) {
	my( $arg, $sep ) = ($1,$2,$');
	$str = $';

	if( $sep =~ /[\s\t]/ ) {
	    push( @res, $buff.$arg );
	    $buff = "";
	}
	else {
	    $buff .= $arg;
	    if( $str =~ /^(.*?)\Q$sep\E/ ) {
		$buff .= $1;
		$str = $';
	    }
	    else {
		$buff .= $str;
		$str = "";
	    }
	    if( $str =~ /^[\s\t]+/ ) {
		$str = $';
		push( @res, $buff );
		$buff = "";
	    }

	}
    }
    $str = $buff.$str;
    push( @res, $str ) if( $str );
    return @res;
}


1;

